/**
 * Extension messaging
 */
var messaging = {
    init: function () {
        this.onMessage();

        //check for ref deletion and notify extension and CS to delete it
        var deletedRefId = $("[data-ndl-deleted-ref-id]").data('ndl-deleted-ref-id');
        if (deletedRefId && utils.isNumeric(deletedRefId)) {
            utils.debug('Delete ref id is ', deletedRefId);
            var body = {
                id_ref: deletedRefId
            };
            var message = {
                subject: 'on_ref_deleted',
                body: body
            };

            messaging.sendMessage(message, function (res) {
                utils.debug(res);
            });
        }
    },
    /**
     * Send message and receive it response here
     */
    sendMessage: function (message, callback) {
        chrome.runtime.sendMessage(message, function (response) {
            utils.debug('receive response ', response);
            callback && callback(response);
        });
    },
    //Incomming messages
    onMessage: function () {
        chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {

            utils.debug('Incomming message ', request, sender);
            var subject = request.subject;
            if (subject && processIncomingCsMessages[subject]) {
                processIncomingCsMessages[subject](request, sender, sendResponse);
            }
        });
    },
};
/**
 * process incoming messages
 */
var processIncomingCsMessages = {
    /**
     * Update ref legende only on current user fil
     * (unique idRef is used)
     */
    update_legende: function (req, sender, sendResp) {
        var body = req.body,
            $ifrFilBody = content._$iframeFilBody;
        if (null === $ifrFilBody) {
            return;
        }
        var idRef = body.id,
            $fil = $ifrFilBody.find('.fil'),
            $slide = $fil.find('[data-id=' + idRef + ']');

        utils.debug(req, $slide);
        if ($slide.length > 0) {
            var $header = $slide.find('.header').addClass('contain-legende');
            var $legende = $header.find('.legende');
            if($legende.length === 0){
                $legende = '<div class="legende clamp"><div class="content"></div></div>';
                $header.append($legende);
            }
            $header.find('.legende .content')
                .html(utils.autoLinkHashtags(body.legende));          
            // $header.find('.legende .content')
            //     .text(body.legende);
        }
    },
    /**
     * Live update slider dates only on current user fil when ref is added
     */
    update_slider_dates: function (req, sender, sendResp) {
        var body = req.body,
            currentUserCle = body.user_cle,
            dates = body.dates,
            swiper = sliderBarCs._swiper;

        if (null === swiper) {
            return;
        }
        var userCle = swiper.params.user.cle;
        if (!dates || dates.length === 0 || userCle !== currentUserCle) {
            return;
        }
        var $ifrFilBody = content._$iframeFilBody,
            $fil = $ifrFilBody.find('.fil');

        if ($fil.length === 0) {
            return;
        }
        utils.debug('live update slider dates', currentUserCle, dates);
        var activeIndx = swiper.realIndex;
        var idInitialRef = $(swiper.slides[activeIndx]).data('id');
        sliderBarCs.init(dates, idInitialRef, swiper);
    },
    /**
     * remove delete ref from swiper if present and 
     * send new active ref data to bg if the deleted was active
     */
    remove_deleted_ref_slide: function (req, sender, sendResp) {
        var body = req.body,
            idRef = body.id_ref,
            swiper = content.getSwiperInstance();
        if (null === swiper) {
            return;
        }
        var $swiperContainer = $(swiper.el),
            $slide = $swiperContainer.find('[data-id=' + idRef + ']'),
            idxRef = $slide.index();
        if (idxRef < 0) {
            return;
        }
        swiper.removeSlide(idxRef);
        swiper.update();
        sliderBarCs.removeSliderRef(idRef);
        utils.debug('slide removed ');
        if (!$slide.hasClass('swiper-slide-active')) {
            return;
        }

        var $newActiveSlide = $(swiper.slides[idxRef + 1] || swiper.slides[idxRef - 1]);
        var slideData = $newActiveSlide.data();
        body = {
            ref: {
                idRef: slideData.id,
                linkId: slideData.linkId
            }
        };
        sendResp({
            subject: req.subject,
            body: body
        });
    },
    /**
     * Fil data is changed in extension
     */
    notify_data_changed: function (req, sender, sendResp) {
        var data = req.body,
            what = req.what;
        switch (what) {
            case 'memorized-ref':
                utils.debug('memorized-ref changed ', data);
                var user = content.activeUser;
                if (user.cle === data.userCle) {
                    var isMemorized = data.isMemorized;
                    if (isMemorized) {
                        sliderBarCs.setSliderMarker(data);
                    } else {
                        sliderBarCs.hideMarker();
                    }
                    content.setMemorizeRefSate(isMemorized, data);
                    content.setFilColor(data.filColors, data.userCle);
                }
                break;
        }
    }
};

var contentTemplate = {
    templateSlide: function (dataTemplate) {
        var data = dataTemplate ? dataTemplate : {},
            legende = data.legende ? utils.autoLinkHashtags(data.legende) : data.legende,
            createdAt = utils.slideShortDate(data.createdAt),
            localRef = data.localRef,
            title = data.title,
            domain = data.domain,
            titleDate = utils.toISOString(data.createdAt);

        var t = `<li class="swiper-slide"><div class="slide-inner">
            <div class="slide-content">
                <div class="top-layout">
                    <div class="date" title=${titleDate}>${createdAt}</div>
                    <div class="ref-count">${localRef}</div>
                </div>
                <div class="separator"></div>
                <div class="layout">
                    <div class="header">
                        <div class="link-title"><span class="title">${title}</span></div>
                        <div class="legende clamp"><div class="content">${legende}</div></div>
                    </div>
                    <div class="footer">
                        <div class="domain">${domain}</div>
                        <div class="memorise-fil-ref" data-title="Mémoriser ce fil depuis cette référence">
                            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 100 100">
                                <path d="M78.67 95V11.17A6.16 6.16 0 0 0 72.51 5h-45a6.16 6.16 0 0 0-6.16 6.16V95L50 80z"></path>
                            </svg>
                        </div>
                    </div>
                </div>
            </div>
        </div></li>`;
        return $(t);
    },
    sliderInner: function () {
        var $template =
            $(`<div class="slider-info-inner">
            <div class="slider-date slider-start-date"></div>
            <div class="slider-wraper">
                <div class="slider-container">
                    <div id="slider"></div>
                    <div class="marker-slider marker-memorized">
                        <button class="marker-btn">
                            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 100 100">
                                <path d="M78.67 95V11.17A6.16 6.16 0 0 0 72.51 5h-45a6.16 6.16 0 0 0-6.16 6.16V95L50 80z"></path>
                            </svg>
                        </button>
                    </div> 
                </div>
            </div>   
            <div class="slider-date slider-end-date"></div>
        </div>`);

        return $template;
    },
    filMenu: function () {
        var t =
            `<div class="fil-title">
                <div class="title">Fil sans titre</div>
                <div class="close-fil-menu"></div>
            </div>
            <div class="fil-actions">
                <div class="action-left">
                    <div class="action">
                        <div class="ndl-svg-button-container">
                        <button class="button svg-button btn-contact" data-action="contact">
                                <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 1792 1792">
                                    <path d="M1664 1504V736q-32 36-69 66-268 206-426 338-51 43-83 67t-86.5 48.5T897 1280h-2q-48 0-102.5-24.5T706 1207t-83-67q-158-132-426-338-37-30-69-66v768q0 13 9.5 22.5t22.5 9.5h1472q13 0 22.5-9.5t9.5-22.5zm0-1051v-24.5l-.5-13-3-12.5-5.5-9-9-7.5-14-2.5H160q-13 0-22.5 9.5T128 416q0 168 147 284 193 152 401 317 6 5 35 29.5t46 37.5 44.5 31.5T852 1143t43 9h2q20 0 43-9t50.5-27.5 44.5-31.5 46-37.5 35-29.5q208-165 401-317 54-43 100.5-115.5T1664 453zm128-37v1088q0 66-47 113t-113 47H160q-66 0-113-47T0 1504V416q0-66 47-113t113-47h1472q66 0 113 47t47 113z"/>
                                </svg>
                                <span class="svg-text">Contacter</span>
                            </button>
                        </div>
                    </div>
                </div>
            </div>`;

        return t;
    }

};
var content = {
    _swiper: null,
    _$iframeFil: null,
    _$iframeFilBody: null,
    _$iframeFAB: null,
    activeUser: null,
    currentUser: null,
    /**
     * initialize content by sending a init message to bg
     */
    initialize: function () {
        var message = {
            subject: 'init_frame'
        };
        var self = this;
        utils.debug('Initialize : Loading fil data...');
        messaging.sendMessage(message, function (response) {
            if (!response) {
                utils.debug('No response : no data received');
                return;
            }
            utils.debug('data received : ', response);
            var body = response.body;

            if (!body.isLogged) {
                utils.debug('not logged');
                //TODO show message not logged
                return;
            }
            var fil = body.fil,
                tabId = body.tabId;

            if (!fil || !fil.refs || fil.refs.length === 0) {
                utils.debug('No references ', fil);
                return;
            }
            utils.debug('Initializing frames ...');
            self.initIframes(function ($ifrFil, $ifrFAB) {

                //Wait a momment to init otherwise FF bug
                setTimeout(function () {
                    var currentUserData = body.currentUser,
                        currentUser = currentUserData.user;
                    var $ifrBody = $ifrFil.contents().find('body');
                    var $swiperContainer = $ifrBody.find('.swiper-container');
                    self._$iframeFil = $ifrFil;
                    self._$iframeFilBody = $ifrBody;
                    self._$iframeFAB = $ifrFAB;
                    var swiper = self.createSwiper(fil.user.cle, $swiperContainer);
                   
                    if (null === swiper) {
                        utils.debug('ERR_CREATE_SWIPER');
                        return null;
                    }
                    utils.debug('All Frames && swiper initialized !!');
                    self._swiper = swiper;
                    self.initializeSwiper(fil, currentUser);
                    self.filMenu(fil.user, currentUser, fil.is_following);
                    self.openCloseFilButton(tabId);
                }, 400);
            });
        });
    },
    createIframe: function () {
        return $('<iframe></iframe>', {
            scrolling: 'no',
            frameborder: 0,
            allowtransparency: true,
            class: 'ndl-iframe'
        });
    },
    /**
     * load fil iframe css
     */
    addStyleSheets: function ($iframe) {
        var styles = [
            'scripts/vendor/css/swiper.min.css',
            'scripts/vendor/css/nouislider.min.css',
            'content/css/topbar.css',
            'content/css/content-fil.css'
        ];
        var cssLinks = '';
        for (var i = 0; i < styles.length; i++) {
            cssLinks += '<link rel="stylesheet" href= "' +
                chrome.runtime.getURL(styles[i]) + '" />';
        }
        var $ifrHead = $iframe.contents().find('head');
        $ifrHead.append(cssLinks);
    },
    /**
     * initialize all iframes (iframe fil and iframe fil closer) and pass them
     * to callback
     */
    initIframes: function (cb) {
        var self = this;
        setTimeout(function () {
            self.initFilIframe(function ($ifrFil) {
                self.initFABiframe(function ($ifrFAB) {
                    cb($ifrFil, $ifrFAB);
                });
            });
        }, 400);
    },
    /**
     * Add iframe to current page and init it html body, then call callback
     * when it is ready
     */
    initFilIframe: function (cb) {
        var self = this;
        //clear first to be sure no dup
        $('#ndl-topbar-iframe').remove();
        var $iframe = self.createIframe();
        $iframe.attr('id', 'ndl-topbar-iframe');
        $('body').append($iframe);
        utils.debug('init fil frame ...');
        $iframe.ready(function () {
            self.addStyleSheets($iframe);
            var $body = $iframe.contents().find('body');
            var $menuFil = $('<div class="fil-menu"></div>');
            var $filContainer =
                $(`<div class="fil-container">
                        <div class="fil">
                            <div class="fil-body segment">
                                <div class="swiper-container">
                                    <ul class="swiper-wrapper"></ul>
                                    <div class="swiper-button swiper-button-prev"></div>
                                    <div class="swiper-button swiper-button-next"></div>
                                </div>
                                <div class="slider-info"></div>
                            </div>
                        </div>
                    </div>`);

            var $page =
                $(`<div class="page-content">
                    <div class="big loader page-content-preloader"></div>
                    <div class="btn-close-fil"></div>
                    <div class="btn-fil-menu"></div>
                </div>`);

            $page.append([$menuFil, $filContainer]);
            $body.append($page);
            utils.debug('Fil frame ready !');
            cb($iframe);
        });
    },
    /**
     * Add iframe FAB (Floating Action Button) to current page 
     * and init it html body, then call callback
     * when it is ready
     */
    initFABiframe: function (cb) {
        utils.debug('init FAB frame...');
        var $ifrFAB = this.createIframe();
        $ifrFAB.attr({
            "id": "ndl-fab-iframe"
        });
        $('body').append($ifrFAB);

        $ifrFAB.ready(function () {
            var $ifrBody = $ifrFAB.contents().find('body');
            var $ifrHead = $ifrFAB.contents().find('head');
            var cssLinks = '<link rel="stylesheet" href= "' +
                chrome.runtime.getURL('content/css/ndl-button-fab.css') + '" />';
            var iconPlus = '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 100 100"><path d="M55.625 5h-11.25v39.375H5v11.25h39.375V95h11.25V55.625H95v-11.25H55.625z"/></svg>';

            var $btnClose = $('<div class="ndl-fab"><div class="wrap">' +
                iconPlus + '</div></div>');
            var $page = $('<div class = "page-content"></div>');

            $ifrHead.append(cssLinks);
            $page.append($btnClose);
            $ifrBody.append($page);
            utils.debug('FAB frame ready !');
            cb($ifrFAB);
        });



    },
    openCloseFilButton: function (tabId) {
        var key = tabId.toString();
        //init iframes states
        chrome.storage.local.get(key, function (result) {
            //result is object with key and value ex {'tabid': {state: 'open'}}
            result = result || {};

            if (!result[key] || (result[key].state === 'open')) {
                var dataFrame = {
                    [key]: {
                        state: 'open'
                    }
                };
                chrome.storage.local.set(dataFrame);
                $ifrFil.addClass('open');
                $ifrFAB.removeClass('open');
            } else {
                $ifrFAB.addClass('open');
                $ifrFil.removeClass('open');
            }
        });

        var $ifrFil = this._$iframeFil,
            $ifrFilBody = this._$iframeFilBody,
            $ifrFAB = this._$iframeFAB,
            $btnCloseFil = $ifrFilBody.find('.btn-close-fil');

        var $fab = $ifrFAB.contents().find('body .ndl-fab');
        $btnCloseFil.click(function () {
            $ifrFil.removeClass('open');
            setTimeout(function () {
                $ifrFAB.addClass('open');
            }, 900);
            var dataFrame = {
                [key]: {
                    state: 'close'
                }
            };
            chrome.storage.local.set(dataFrame);
        });

        $fab.click(function () {
            $ifrFAB.removeClass('open');
            setTimeout(function () {
                $ifrFil.addClass('open');
            }, 400);
            var dataFrame = {
                [key]: {
                    state: 'open'
                }
            };
            chrome.storage.local.set(dataFrame);
        });
        //dragg iframe FAB
        // $fab.mousedown(function (e) {
        //     draggFAB.fabStartMoving($ifrFAB, e);
        // });

    },
    /**
     * Fil menu content
     */
    filMenu: function (user, currentUser, isFollowing) {
        if (user.cle === currentUser.cle) {
            this._$iframeFilBody.find('.btn-fil-menu').remove();
            return;
        }
        var $ifrFilBody = this._$iframeFilBody,
            $btnMenuFil = $ifrFilBody.find('.btn-fil-menu'),
            $filMenu = $ifrFilBody.find('.fil-menu');

        $filMenu.html(contentTemplate.filMenu());
        if (!user.has_email) {
            $filMenu.find('.btn-contact').closest('.action').remove();
        }
        $filMenu.find('.title').text(user.title);

        $btnMenuFil.click(function () {
            $filMenu.addClass('active');
        });

        $filMenu.find('.close-fil-menu').click(function () {
            $filMenu.removeClass('active');
        });

        $filMenu.find('.fil-actions .action-left button')
            .click(function () {
                var $this = $(this);
                var message = {};
                switch ($this.data('action')) {
                    case 'contact':
                        message = {
                            subject: 'contact_request',
                            body: {
                                user: user
                            }
                        };
                        messaging.sendMessage(message, function (res) {
                            utils.debug(res);
                        });
                        break;
                }

            });
    },
    /**
     * placeholder data loading
     */
    contentPlaceholder: function () {
        var $template = contentTemplate.templateSlide();

        $template.find('.legende, .memorise-fil-ref').remove();
        $template.find('.date, .ref-count, .title, .domain').empty();
        $template.find('.date, .ref-count, .domain').addClass('holder');
        var holder, str;
        holder = str = '<div class="holder"></div>';
        for (var i = 0; i < 3; i++) {
            str += holder;
        }
        $template.find('.title').html(str);

        $template.find('.slide-content').addClass('placeholder');
        $template.addClass('placeholder-slide');
        return $template;
    },
    /**
     * Create content swiper
     */
    createSwiper: function (cleUser, $swiperContainer) {
        utils.debug('initializing swiper in container ', $swiperContainer);
        var swiper = new Swiper($swiperContainer, {
            init: false,
            cleUser: cleUser,
            slidesPerView: 'auto',
            spaceBetween: 45,
            centeredSlides: true,
            slideToClickedSlide: true,
            // roundLengths: true,
            simulateTouch: false,
            keyboard: {
                enabled: true,
            },
            on: {
                slideChangeTransitionStart: function () {
                    content.loadMoreRefs(this);
                    if (this.slides.length > 2) {
                        var $swiperContainer = (this.$el);
                        $swiperContainer.find('.swiper-button').addClass('ndl-active');

                    }
                },
            }
        });
        //swiper is not init if it has not prop slides
        return swiper.slides ? swiper : null;
    },
    /**
     * return swiper instance 
     */
    getSwiperInstance: function () {
        var $ifrBody = content._$iframeFilBody;
        if (null === $ifrBody) {
            utils.debug('ifr body is null');
            return null;
        }

        var $swiperContainer = $ifrBody.find('.fil .swiper-container');
        if ($swiperContainer.length === 0) {
            return null;
        }
        var swiper = $swiperContainer[0].swiper;
        return swiper.slides ? swiper : null;
    },
    /**
     * Set appropriate fil color class according color priorities 
     * and remove other color classes
     */
    setFilColor: function (colorsPriorities, userCle) {
        var self = this,
            message = {
                subject: 'load_history',
                body: null
            };
        messaging.sendMessage(message, function (res) {
            if (!res) {
                utils.debug('No response ');
                return;
            }
            var history = res.body;
            var colorData = utils.getFilColor(colorsPriorities, userCle, history);
            if (colorData.color) {
                var classes = utils.filColorsClasses;
                self._$iframeFilBody.find('.fil, .fil-menu')
                    .removeClass(classes)
                    .addClass(colorData.color);
            }
        });
    },
    /**
     * Initialze swiper with fil data
     */
    initializeSwiper: function (filData, currentUser) {
        utils.debug('initi swiper data ', filData);
        var cleUser = filData.user.cle,
            refs = filData.refs,
            swiper = this._swiper,
            activeRef = filData.activeRef;

        swiper.params.user = filData.user;
        swiper.params.currentUser = currentUser;
        swiper.params.filParams = {
            firstRefLinkId: filData.first_ref_link_id,
            lastRefLinkId: filData.last_ref_link_id,
            refCount: filData.ref_count
        };

        var $swiperContainer = $(swiper.el),
            $fil = $swiperContainer.closest('.fil');

        $fil.addClass('fil').addClass('fil-' + cleUser);
        this.activeUser = filData.user;
        this.currentUser = currentUser;

        var $ul = $('<ul></ul>');
        for (var i = 0, $slide; i < refs.length; i++) {
            $slide = this.createSlide(refs[i]);
            //prepend to create child in good order
            $ul.prepend($slide);
        }
        swiper.prependSlide($ul.children());

        if (filData.user.cle === currentUser.cle) {
            $fil.addClass('teal');
        }

        if (filData.ref_count > 2) {
            // show navigation btn
            $swiperContainer.find('.swiper-button').addClass('ndl-active');

            $swiperContainer.find('.swiper-button-prev').click(function () {
                swiper.slidePrev();
            });

            $swiperContainer.find('.swiper-button-next').click(function () {
                swiper.slideNext();
            });
        }

        if (filData.ref_count > 10) {
            var body = {
                user_cle: cleUser
            };
            var message = {
                subject: 'load_fil_dates',
                body: body
            };

            messaging.sendMessage(message, function (res) {
                utils.debug(res);
                if (!res) {
                    utils.debug('No response ');
                    return;
                }
                var body = res.body;
                if (body.error) {
                    return;
                }
                var dates = body.dates;
                sliderBarCs.init(dates, activeRef.idRef, swiper);

            });
        }
        //init now swiper to bind events and validate initialSlide
        swiper.init();
        var idRef = activeRef.idRef;
        var initialSlide = $swiperContainer.find('[data-id=' + idRef + ']').index();
        swiper.update();
        swiper.slideTo(initialSlide, 0);

        if (filData.followedRef && filData.followedRef.id) {
            this.setMemorizeRefSate(true, filData.followedRef);
        } else {
            //to clear data memorized ref in data fil too
            this.setMemorizeRefSate();
        }

        this.setFilColor(filData.fil_colors, cleUser);
    },
    /**
     * Create a swiper slide and add events
     */
    createSlide: function (ref) {
        //minus 1 for visible ref
        var localRef = ref.link.local_ref - 1;
        //var isCurrentUser = ref.user_cle === currentUserCle;

        var dataTemplate = {
            legende: ref.legende,
            createdAt: ref.created_at,
            localRef: localRef,
            title: ref.link.title,
            domain: utils.hostname(ref.link.url),
        };
        var $slide = contentTemplate.templateSlide(dataTemplate);

        var $slideContent = $slide.find('.slide-content');
        $slide.attr({
            'data-id': ref.id,
            'data-link-id': ref.link_id,
            'data-url': ref.link.url
        });
        // =========legende========
        if (null === ref.legende || ref.legende.trim().length === 0) {
            $slide.find('.legende').remove();
        } else {
            $slide.find('.header').addClass('contain-legende');
        }
        // ========counter===========
        if (localRef <= 0) {
            $slide.find('.ref-count').remove();
        }

        var userCross = ref.crossing,
            self = this,
            $fil = self._$iframeFilBody.find('.fil'),
            currentUser = this.currentUser,
            isCurrentUser = currentUser.cle === ref.user_cle;

        if (userCross) {
            $slideContent.addClass('cross');
        }
        if (isCurrentUser) {
            $slideContent.find('.memorise-fil-ref').remove();
        }


        if (ref.memorized) {
            $fil.addClass('following');
            $slideContent.addClass('memorized')
                .find('.memorise-fil-ref')
                .attr('data-title', NDLConst.TITLE_REF_MEMORIZED);
        }


        $slideContent.click(function () {
            //TO DO prevent multiple send message to browse fil
            content.browseFil(ref);
        });
        $slideContent.find('.memorise-fil-ref').click(function (e) {
            e.stopPropagation();
            var isMemorized = $slideContent.hasClass('memorized');
            var body = {
                id_ref: ref.id,
                link_id: ref.link_id,
                user_cle: ref.user_cle,
                method: isMemorized ? 'unfollow' : 'follow'
            };
            var message = {
                subject: 'follow_unfollow',
                body: body
            };
            //memorize/unmemorize this ref
            $fil.addClass('memorizing');
            messaging.sendMessage(message, function (res) {
                $fil.removeClass('memorizing');
                if (!res) {
                    //No response
                    return;
                }
                var body = res.body;
                if (body.error) {
                    return;
                }
                var dataRef = {
                    id: ref.id,
                    link_id: ref.link_id,
                    user_cle: ref.user_cle,
                    fil_colors: body.fil_colors
                };
                self.setMemorizeRefSate(body.is_following, dataRef);
                self.setFilColor(body.fil_colors, ref.user_cle);
                if (body.is_following) {
                    sliderBarCs.setSliderMarker(dataRef);
                } else {
                    sliderBarCs.hideMarker();
                }
                //notify fil to update memorised ref
                dataRef.isMemorized = body.is_following;
                utils.debug('Memorised ref state changed', dataRef);
                message = {
                    subject: 'on_memorized_ref_changed_from_cs',
                    body: dataRef
                };
                messaging.sendMessage(message);
            });

        });

        return $slide;
    },
    /**
     * Add slides to swiper and replace the loader
     * @param {String} direction prev or next 
     * @param {Array} arraySlides html elements 
     * @param {Object} swiper 
     */
    addSlides: function (arraySlides, swiper, direction) {
        var func, indexPlaceholder;
        if (direction === 'prev') {
            indexPlaceholder = 0;
            func = 'prependSlide';
        } else if (direction === 'next') {
            indexPlaceholder = swiper.slides.length - 1;
            func = 'appendSlide';
        } else {
            swiper.removeAllSlides();
            swiper.appendSlide(arraySlides);
            swiper.update();
            return;
        }
        var $placeholder = $(swiper.slides[indexPlaceholder]);

        setTimeout(function () {
            if ($placeholder.hasClass('placeholder-slide')) {
                var loaderReplacer = arraySlides.shift();
                $placeholder.replaceWith(loaderReplacer);
                swiper.update();
            }
            if (arraySlides.length === 0) {
                return;
            }
            swiper[func](arraySlides);
            swiper.update();
        }, 800);
    },
    /**
     * Navigate to url of clicked ref
     * @param {Object} ref the clicked ref
     */
    browseFil: function (ref) {
        var body = {
            user_cle: ref.user_cle,
            link_id: ref.link_id,
            id_ref: ref.id,
        };
        var message = {
            subject: 'browse_content_fil',
            body: body
        };

        messaging.sendMessage(message, function (res) {
            var url = ref.link.url;
            window.location.href = url;
        });

    },
    /**
     * Add more refs (prev or next) to the swiper 
     */
    loadMoreRefs: function (swiper) {

        var realIndex = swiper.realIndex,
            previousIndex = swiper.previousIndex,
            direction = realIndex < previousIndex ? 'prev' : 'next',
            linkId, idxPlaceholder, swiAddFunc,
            totalSlides = swiper.slides.length;

        if (direction === 'prev') {
            if ((totalSlides > 5) && realIndex > 1) {
                //abort loading
                return;
            }
            idxPlaceholder = 0;
            swiAddFunc = 'prependSlide';
        } else {
            if ((totalSlides > 5) && realIndex < (totalSlides - 2)) {
                //abort loading
                return;
            }
            idxPlaceholder = totalSlides - 1;
            swiAddFunc = 'appendSlide';
        }
        if ($(swiper.slides[idxPlaceholder]).hasClass('placeholder-slide')) {
            return;
        }
        linkId = $(swiper.slides[idxPlaceholder]).data('link-id');
        var filParams = swiper.params.filParams;
        if (linkId === filParams.firstRefLinkId) {
            //this is the first ref
            return;
        }
        //avoid add remove place holder if few slides
        var $placeHolder;
        if (totalSlides > 5) {
            $placeHolder = this.contentPlaceholder();
            swiper[swiAddFunc]($placeHolder);
        }

        var user = swiper.params.user;
        // var currentUser = swiper.params.currentUser;

        var params = {
            method: direction,
            user_cle: user.cle,
            link_id: linkId
        };

        var message = {
            subject: 'load_more_refs',
            body: params
        };

        messaging.sendMessage(message, function (res) {
            utils.debug(res);
            if (!res || res.body.error || res.body.refs.length === 0) {
                if ($placeHolder) {
                    idxPlaceholder = direction === 'prev' ? idxPlaceholder : idxPlaceholder + 1;
                    swiper.removeSlide(idxPlaceholder);
                }
            }
            if (!res) {
                utils.debug('No response ');
                return;
            }
            var data = res.body;
            if (data.error) {
                //TO DO: show message (even ERR_END_FIL)
                return;
            }

            if (data.refs.length > 0) {
                var refs = data.refs;
                var $ul = $('<ul></ul>');
                for (var i = 0, $slide; i < refs.length; i++) {
                    $slide = content.createSlide(refs[i]);
                    $ul.append($slide);
                }
                var slides = $ul.children();
                //will remove placeholder
                content.addSlides(slides.toArray(), swiper, direction);
            }

        });
    },
    setMemorizeRefSate: function (isMemorized, ref) {
        var $ifrBody = this._$iframeFilBody;
        var $fil = $ifrBody.find('.fil'),
            idRef = ref ? ref.id : null,
            $slideContent = $fil.find(
                '.swiper-slide[data-id="' + idRef + '"] .slide-content');
        if (isMemorized) {
            $fil.addClass('following');
            $fil.find('.slide-content').removeClass('memorized');
            $slideContent.addClass('memorized')
                .find('.memorise-fil-ref')
                .attr('data-title', NDLConst.TITLE_REF_MEMORIZED);
        } else {
            $fil.removeClass('following');
            $fil.find('.slide-content').removeClass('memorized');
            $slideContent.find('.memorise-fil-ref')
                .attr('data-title', NDLConst.TITLE_REF_NOT_MEMORIZED);
            $fil.removeData('memorized-ref');
        }
        //add it for slider marker 
        if (idRef && isMemorized) {
            $fil.data('memorized-ref', ref);
        }
    },
};



/**
 * slider bar manager
 */

var sliderBarCs = {
    $tooltipRefCount: $('<div class="tooltip-slider tooltip-slider-refs"></div>'),
    $tooltipDate: $('<div class="tooltip-slider tooltip-slider-date"></div>'),
    $startDate: null,
    $endDate: null,
    $sliderInfo: null,
    _swiper: null,
    _slider: null,
    _dates: null,
    init: function (dates, idInitialRef, theSwiper) {
        var $filContainer = $(theSwiper.$el).closest('.fil'),
            $sliderInfo = $filContainer.find('.slider-info'),
            slider = $sliderInfo.find('#slider')[0],
            initial = utils.findIndexRef(dates, 'id', idInitialRef),
            self = this,
            $iframeFil = content._$iframeFil;

        $iframeFil.addClass('slider-dates');
        // return
        if (!slider) {
            $sliderInfo.append(contentTemplate.sliderInner());
            slider = $sliderInfo.find('#slider')[0];
            var sliderMax = dates.length - 1;

            noUiSlider.create(slider, {
                start: initial,
                animate: false,
                behaviour: 'tap-hover',
                range: {
                    min: 0,
                    max: sliderMax
                }
            });
            var $handle = $sliderInfo.find('.noUi-handle');
            $handle.append([this.$tooltipRefCount, this.$tooltipDate]);

            this.$startDate = $sliderInfo.find('.slider-start-date');
            this.$endDate = $sliderInfo.find('.slider-end-date');

            this._swiper = theSwiper;
            this._slider = slider;
            this.$sliderInfo = $sliderInfo;
            var timeout;
            //add transition event on swiper to sync with slider
            theSwiper.on('transitionStart', function () {
                clearTimeout(timeout);
            });
            theSwiper.on('transitionEnd', function () {
                //update handle after slider is completely stoped
                timeout = setTimeout(function () {
                    self.updateHandlePosition();
                }, 10);
            });
        } else {
            var newOptions = {
                range: {
                    min: 0,
                    max: dates.length - 1
                },
                start: initial
            };
            slider.noUiSlider.off();
            slider.noUiSlider.updateOptions(newOptions);
        }

        var startedIdx = -1,
            stopedIdx = initial;
        this._dates = dates;
        this.updateSliderUI(initial);

        $sliderInfo.find('.slider-start-date, .slider-end-date')
            .off('click')
            .click(function () {
                var $this = $(this),
                    currentIdx = 0,
                    currentHandlePos = slider.noUiSlider.get();

                currentHandlePos = Math.floor(currentHandlePos);
                if ($this.hasClass('slider-end-date')) {
                    currentIdx = dates.length - 1;
                }
                if (currentIdx === currentHandlePos) {
                    return;
                }
                self.setHandlePos(currentIdx);
                self.sliderPositionChanged(currentIdx);
            });

        slider.noUiSlider.on('update', function (values, handle) {
            var currentIdx = Math.floor(values[handle]);
            self.updateSliderUI(currentIdx);
        });

        slider.noUiSlider.on('start', function (values, handle) {
            startedIdx = Math.floor(values[handle]);
        });

        slider.noUiSlider.on('change', function (values, handle, unencoded, tap) {
            var currentIdx = Math.floor(values[handle]);
            stopedIdx = currentIdx;
            if (startedIdx >= 0 && stopedIdx >= 0 && (startedIdx === stopedIdx) && !tap) {
                //do not get data for same idx
                return;
            }
            self.sliderPositionChanged(currentIdx);
            startedIdx = currentIdx;

        });

        //add memorized marker
        var $fil = this.$sliderInfo.closest('.fil'),
            ref = $fil.data('memorized-ref');
        if ($fil.hasClass('following') && ref && ref.id) {
            this.setSliderMarker(ref);
        }

    },
    /**
     * update slider dates and refs count values
     */
    updateSliderUI: function (handlePos) {
        var dates = this._dates,
            len = dates.length;
        var startDate = utils.sliderDate(dates[0].created_at),
            endDate = utils.sliderDate(dates[len - 1].created_at),
            handleDate = utils.sliderDate(dates[handlePos].created_at),
            handleRefCount = (handlePos + 1) + ' / ' + len;

        this.$startDate.text(startDate);
        this.$endDate.text(endDate);
        this.$tooltipRefCount.text(handleRefCount);
        this.$tooltipDate.text(handleDate);
    },
    /**
     * Sync swiper and and handle when swiper move
     */
    updateHandlePosition: function () {
        var swiper = this._swiper;
        var realIndex = swiper.realIndex,
            activeSlide = $(swiper.slides[realIndex]),
            idRef = activeSlide.data('id'),
            dates = this._dates,
            currentHandlePos = this._slider.noUiSlider.get();

        currentHandlePos = Math.floor(currentHandlePos);
        if (!idRef || !dates[currentHandlePos]) {
            return;
        }
        if (idRef === dates[currentHandlePos].id) {
            return;
        }
        var handlePos = utils.findIndexRef(dates, 'id', idRef);
        this.setHandlePos(handlePos);
    },
    /**
     * move handle to position with animation when not tap
     */
    setHandlePos: function (handlePos) {
        if (handlePos < 0) {
            return;
        }
        var $slider = $(this._slider);
        $slider.addClass('noUi-state-tap');
        this._slider.noUiSlider.set(handlePos);
        setTimeout(function () {
            $slider.removeClass('noUi-state-tap');
        }, 700);
    },
    /**
     * Handle released fetch data corresponding to this position
     */
    sliderPositionChanged: function (currentIdx) {
        var idRef = this._dates[currentIdx].id,
            dateRef = this._dates[currentIdx].created_at,
            swiper = this._swiper,
            $swiperContainer = $(swiper.el),
            $newActiveSlide = $swiperContainer.find('[data-id=' + idRef + ']');
        var idxRef = $newActiveSlide.index();

        if (idxRef > -1) {
            return swiper.slideTo(idxRef);
        }

        var params = {
            user_cle: this._swiper.params.cleUser,
            id_ref: idRef,
            date_ref: dateRef,
        };
        var message = {
            subject: 'slider_handle_drag',
            body: params
        };
        var $filBody = $(this._swiper.$el).closest('.fil-body');
        $filBody.addClass('loading');
        messaging.sendMessage(message, function (res) {
            $filBody.removeClass('loading');
            if (!res) {
                utils.debug('No response ');
                return;
            }
            var data = res.body;
            if (data.error) {
                //TO DO: show message (even ERR_END_FIL)
                return;
            }
            if (data.refs.length > 0) {
                var refs = data.refs;
                var $ul = $('<ul></ul>');
                for (var i = 0, $slide; i < refs.length; i++) {
                    $slide = content.createSlide(refs[i]);
                    $ul.append($slide);
                }
                var slides = $ul.children(),
                    initialSlide = utils.findIndexRef(refs, 'id', idRef);
                content.addSlides(slides.toArray(), swiper);
                swiper.slideTo(initialSlide);
            }

        });
    },
    /**
     * Remove a ref from slider (e.g.: when a ref is deleted from fil)
     */
    removeSliderRef: function (idRef) {
        var dates = this._dates,
            swiper = this._swiper;
        if (!dates || !swiper) {
            return;
        }
        var idxRef = dates.findIndex(function (ref) {
            return ref.id === idRef;
        });
        if (idxRef < 0) {
            return;
        }
        dates.splice(idxRef, 1);
        this.updateSliderUI(swiper.realIndex);
    },

    /**
     * Add memorized marker point on slider bar,
     * on slider bar to go back easily
     */
    setSliderMarker: function (dataMarker) {
        var dates = this._dates,
            swiper = this._swiper,
            $sliderInfo = this.$sliderInfo;

        if (null === dates || null === swiper || null === $sliderInfo) {
            return;
        }
        var idxRef = dates.findIndex(function (ref) {
            return ref.id === dataMarker.id;
        });
        utils.debug('datamarker', dataMarker, 'idxRef = ', idxRef);
        var $marker = $sliderInfo.find('.marker-memorized');
        if (idxRef < 0) {
            $marker.removeClass('active');
            return;
        }

        var min = 0,
            max = dates.length - 1,
            range = [min, max];
        var percentValue = utils.fromIndexToPercentage(range, idxRef);

        $marker.css({
            left: 'calc(' + percentValue + '% - ' + $marker.width() / 2 + 'px )'
        }).addClass('active');
        var $swiperContainer = $(swiper.el),
            idRef = dates[idxRef].id,
            self = this;

        $marker.off('click').click(function (e) {
            e.stopPropagation();
            var $slide = $swiperContainer.find('[data-id=' + idRef + ']');
            if ($slide.length > 0) {
                var $slideContent = $slide.find('.slide-content');
                //remove clamp for firefox before highlighting
                var $elWithClamps = $slideContent.find('.clamp');
                $elWithClamps.removeClass('clamp');
                var indexSlide = $slide.index();
                swiper.slideTo(indexSlide);
            } else {
                self.sliderPositionChanged(idxRef);
            }
        });
    },
    /**
     * Hide a marker
     */
    hideMarker: function () {
        var $sliderInfo = this.$sliderInfo;
        if (null === $sliderInfo) {
            return;
        }
        var $marker = $sliderInfo.find('.marker-memorized');
        $marker.off('click').removeClass('active');
    },
};


//TO DO: allow dragging FAB
// var draggFAB = {
//     startMoving: function ($ifrFAB, evt) {
//         // utils.debug($ifrFAB, event);
//         evt = evt || window.event;
//         //mouse position at startup
//         var posX = evt.clientX,
//             posY = evt.clientY;
//         utils.debug(posX, posY);

//         $(document).mousemove(function(evt){
//             evt = evt || window.event;
//             //caculate new mouse position
//             // var posX = 

//         });
//     },
//     move: function(){

//     }

// }

$(document).ready(function () {
    utils.debug('=========== doc ready==========');
    //init content script messaging
    messaging.init();
    // content.initialize();
});


window.addEventListener('load', function(){
    utils.debug('=======doc load====');
    content.initialize();
});