var bg = window.chrome.extension.getBackgroundPage();

var templates = {
    filItem: function (templateData, currentUserCle) {
        var filTitle = templateData.fil_title,
            linkTitle = templateData.title,
            legende = templateData.legende,
            domain = utils.hostname(templateData.url),
            createdAt = utils.slideShortDate(templateData.created_at),
            userCle = templateData.user_cle,
            refCount = templateData.ref_count,
            userInitials = utils.getInitials(filTitle);
        var t = `<div class="column">
                <a class="fil"><div class="fil-content">
                    <div class="fil-header">
                        <div class="media media-left">
                            <div class="user-initials">
                                <div class="initials">${userInitials}</div>
                            </div>
                        </div>
                        <div class="media media-center">
                            <span class="title-text">
                                ${filTitle} <i class="la la-external-link icon-link"></i>
                            </span>
                        </div>
                        <div class="media media-right">
                            <h2>
                                <strong>${refCount}</strong>
                            </h2>
                            <small>Référence${refCount > 1? 's': ''}</small>
                        </div>
                    </div>
                    <div class="last-ref">
                        <div class="content">
                            <div class="last-ref-icon"><i class="la la-rss"></i></div>
                            <div class="ref-description">
                                <div class="link-title">${linkTitle}</div>
                                <div class="ref-infos">
                                    <div class="legende">${legende}</div>
                                    <div class="container-domain">
                                        <span class="domain">${domain}</span>
                                        <span class="date">${createdAt}</span>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div></a>
            </div>`;
        var $filItem = $(t);
        
        if (userCle === currentUserCle) {
            $filItem.find('.fil').addClass('current-user');
        }
        if (!legende) {
            $filItem.find('.legende').remove();
        }
        $filItem.find('.fil .last-ref .content').click(function () {
            var ref = {
                user_cle: userCle,
                id: templateData.id_ref,
                link_id: templateData.link_id,
                link: {
                    url: templateData.url,
                }
            };
            commons.browseOrDiscoverFil(ref, 'browse');
        });

        $filItem.find('.fil-header').click(function () {
            var ref = {
                id: templateData.id_ref,
                user_cle: templateData.user_cle,
                link_id: templateData.link_id
            };
            commons.viewFil(ref);
        });
        return $filItem;

    }
};

var groupInfos = {
    updateHeader: function (group) {
        $('#group-header .group-name').text(group.name);
        var totalUsersGroup = group.group_users_count;
        var $memberCount = $('#group-header .members-count');
        var $joinLeaveBtn = $('.join-leave-group-btn');

        changeMemberBtnState(group.is_member, totalUsersGroup);
        var fetch = false;
        var self = this;
        $joinLeaveBtn.off('click').click(function () {
            if (fetch) {
                return;
            }
            var data = {
                method: group.is_member ? 'leave' : 'join',
                id: group.id
            };
            fetch = true;
            bg.groupsManager.api.groupsData(data, function (res) {
                utils.debug(res);
                if (res.error) {
                    fetch = false;
                    $('.ndl-message-error').text(res.message || NDLConst.ERR_GLOBAL);
                    utils.showHideMessage('error');
                    return;
                }

                group.is_member = res.is_member;
                group.group_users_count += res.member_count;
                changeMemberBtnState(res.is_member, group.group_users_count);
                if (group.group_users_count === 0) {
                    fetch = false;
                    $('.fils-container .fils').empty();
                    var $noRefsMsg = $('.no-refs-msg');
                    $noRefsMsg.addClass('ndl-active')
                        .find('.text').html(NDLConst.NO_USER_IN_GROUP);
                    return;
                }
                //update displayed fils
                self.init(function () {
                    fetch = false;
                });

            });
        });

        function changeMemberBtnState(isMember, totalMember) {
            if (isMember) {
                $joinLeaveBtn.addClass('active')
                    .find('.svg-text').text('abonné');
            } else {
                $joinLeaveBtn.removeClass('active')
                    .find('.svg-text').text("s'abonner");
            }
            $memberCount.text(totalMember +
                ' utilisateur' + (totalMember > 1 ? 's' : ''));
        }
        $('#group-header').addClass('ndl-active');
    },
    init: function (cb) {
        var url = window.location.href;
        var parsedUri = utils.parseUri(url);
        var groupId = parsedUri.queryKey.id;
        if (!groupId || !utils.isNumeric(groupId)) {
            return window.location.href = '/groups/groups.html';
        }
        var data = {
            method: 'infos',
            id: groupId
        };
        var $pageContent = $('.page-content');
        $pageContent.addClass('loading');
        var $noRefsMsg = $('.no-refs-msg');
        $noRefsMsg.removeClass('ndl-active');
        var self = this;
        bg.groupsManager.api.groupsData(data, function (res) {
            utils.debug(res);
            $pageContent.removeClass('loading');
            if (res.error) {
                if (res.error === 'NOT_FOUND') {
                    $('#group-header').removeClass('ndl-active');
                    $noRefsMsg.addClass('ndl-active')
                        .find('.text').html(res.message);
                    return;
                }
                $('.page-global-msg')
                    .addClass('error ndl-active')
                    .find('.error-msg').html(res.message || NDLConst.ERR_GLOBAL);
                return;
            }
            var group = res.group;
            var users = res.users;
            self.updateHeader(group);
            var $fils = $('.fils-container .fils');
            $fils.empty();
            if (group.group_users_count === 0) {
                $noRefsMsg.addClass('ndl-active')
                    .find('.text').html(NDLConst.NO_USER_IN_GROUP);
                return;
            } else {
                if (users.length === 0) {
                    $noRefsMsg.addClass('ndl-active')
                        .find('.text').html(NDLConst.NO_REFS_IN_FIL);
                    return;
                }
            }

            var currentUserCle = res.cle;
            
            for (var i = 0; i < users.length; i++) {
                $fils.append(templates.filItem(users[i], currentUserCle));
            }
            cb && cb();
        });
    }
};


$(function () {
    commons.onPageReady(function(){
        groupInfos.init();
    });
});