var bg = window.chrome.extension.getBackgroundPage();

var templates = {
    groupItem: function (templateData) {
        var groupId = templateData.id,
            name = templateData.name,
            totalUsers = templateData.total_users_group,
            lastUpdatedAtGrp = templateData.updated_at;
        var t =
            `<div class="column">
            <a class="group"><span class="group-content">
                    <span class="name">${name}</span>
                    <span class="stats">
                        <span class="members-count"><i class="la la-users"></i>${totalUsers}</span>
                        <span class="group-last-updated-ref"></span>
                    </span>
            </span></a>
        </div>`;

        var $groupItem = $(t);
        if (lastUpdatedAtGrp) {
            lastUpdatedAtGrp = utils.slideShortDate(lastUpdatedAtGrp);
            $groupItem.find('.group-last-updated-ref').append(
                `<span><i class="la la-clock-o"></i>${lastUpdatedAtGrp}</span>`
            );
        }
        $groupItem.click(function () {
            window.location.href = '/groups/group-infos.html?id=' + groupId;

        });
        return $groupItem;
    },
    searchResultItem: function (templateData) {
        var name = templateData.name,
            totalUsers = templateData.total_users_group,
            totalRefs = templateData.group_refs_count,
            text = 'référence';

        text += totalRefs > 0 ? 's' : '';
        var t =
            `<a class="group">
                <span class="group-content">
                    <span class="name">${name}</span>
                    <span class="stats">
                        <span class="members-count"><i class="la la-users"></i>${totalUsers}</span>
                        <span class="group-refs-count">${totalRefs}<br>${text}</span>
                    </span>
                </span>
            </a>`;
        return t;
    }
};

var groups = {
    ready: function(){        
        this.init();
        this.initContactUser();
    },
    initContactUser: function () {
        //do not init in form cotact
        if ($('body.contact-user').length === 0) {
            return;
        }
        var user = bg.groupsManager.getUserToContact();
        utils.debug(user);
        if (!user || !user.cle) {
            var message = `<h3> Aucun utilisateur à contacter ! <br>
                Pour contacter un utilisateur utiliser le boutton 
                <span>contacter</span> depuis son fil </h3>`;
            $('.no-user-to-contact').html(message)
                .addClass('ndl-active');
            return;
        }
        $('.form-contact-container').addClass('ndl-active');
        $('.title-wrapper .title').text(user.title);
        var $contactUserForm = $('#contact-user-form');
        var $errorMsg = $('.ndl-message-error');
        var $successMsg = $('.ndl-message-success');
        var $areaMessage = $('textarea');

        $areaMessage.focusin(function () {
            $contactUserForm.removeClass('error');
        });

        $('.send-btn').click(function () {
            var message = $areaMessage.val();
            if (!message || message.trim().length === 0) {
                $contactUserForm.addClass('error');
                return;
            }
            var $this = $(this);
            $this.addClass('loading');
            var messageData = {
                to: user.cle,
                message: message
            };

            bg.groupsManager.api.contactUser(messageData, function (res) {
                utils.debug(res);
                if (!res.error) {
                    $successMsg.html('Message envoyé.');
                    $contactUserForm.removeClass('error');
                    $areaMessage.val('');
                    utils.showHideMessage('success');
                } else {
                    $errorMsg.text(res.message || NDLConst.ERR_GLOBAL);
                    $contactUserForm.addClass('error');
                    utils.showHideMessage('error');
                }
                $this.removeClass('loading');
            });

        });
    },
    init: function () {
        //do not init in form contact
        if ($('body.groups').length === 0) {
            return;
        }
        var data = {
            method: 'init'
        };
        var $pageContent = $('.page-content');
        $pageContent.addClass('loading');
        var self = this;
        // return $('body').addClass('no-group');
        bg.groupsManager.api.groupsData(data, function (res) {
            utils.debug(res);
            $pageContent.removeClass('loading');
            if (res.error) {
                if (res.error === 'NO_GROUP_FOUND') {
                    self.initSearch();
                    return $('body').addClass('no-group');
                }
                $('.page-global-msg')
                    .addClass('error ndl-active')
                    .find('.error-msg').html(res.message || NDLConst.ERR_GLOBAL);
                return;
            }
            var $groupsContainer = $('.groups-container .groups');
            var userGroups = res.groups;
            for (var i = 0; i < userGroups.length; i++) {
                $groupsContainer.append(templates.groupItem(userGroups[i]));
            }
            self.initSearch();
        });

    },
    initSearch: function () {
        var $searchBarContainer = $('.search-bar-wrapper .search-bar-container');
        var $searchModal = $('.search-modal');

        $searchBarContainer.addClass('ndl-active');
        $('.search-bar-wrapper input').focusin(function () {
            if ($searchModal.modal('is active')) {
                return;
            }
            $searchModal.modal('show');
            $searchBarContainer.detach();
            $searchModal.find('.modal-header').append($searchBarContainer);
        });
        $('.modal-header .close-modal-search').click(function () {
            $searchModal.modal('hide');
        });

        $(document).keyup(function (e) {
            if (e.which === 27) {
                $searchModal.modal('hide');
            }
        });

        var self = this;
        $searchModal.modal({
            inverted: true,
            closable: false,
            onVisible: function () {
                var $autocomplete = $('.search-modal .search-bar input');

                //the autocomplete is already init do nothing
                if ($autocomplete.autocomplete("instance")) {
                    return;
                }
                $autocomplete.keyup(function () {
                    if ($(this).val().trim().length === 0) {
                        $('#search-results .segment-create-group').remove();
                    }
                });
                var $ulSearchResult;
                $autocomplete.autocomplete({
                    //minLength: 3,
                    appendTo: "#search-results",
                    classes: {
                        "ui-autocomplete": "ui three column doubling stackable grid container groups"
                    },
                    create: function (event, ui) {
                        $ulSearchResult = $('#search-results ul.ui-autocomplete');
                    },
                    source: function (req, callback) {
                        var groupName = req.term ? req.term.trim() : '';
                        if (groupName.length === 0) {
                            return;
                        }
                        var data = {
                            method: 'search',
                            name: groupName
                        };
                        bg.groupsManager.api.groupsData(data, function (res) {
                            if (res.error) {
                                $('.ndl-message-error').text(res.message || NDLConst.ERR_GLOBAL);
                                utils.showHideMessage('error');
                                return;
                            }
                            if (res.groups.length === 0) {
                                $ulSearchResult.addClass('no-result');
                                self.groupNotFound(groupName);
                            } else {
                                $ulSearchResult.removeClass('no-result');
                                $('#search-results .segment-create-group').remove();

                            }
                            callback(res.groups);
                        });
                    }
                });
                $autocomplete.data('ui-autocomplete')._renderItem = function (ul, data) {
                    return $('<li class="column">')
                        .data('value', data)
                        .append(templates.searchResultItem(data))
                        .appendTo(ul);
                };
                //disable events autocompletefocus
                $autocomplete.on('autocompletefocus', function (event, ui) {
                    event.preventDefault();
                });
                $autocomplete.on('autocompleteselect', function (event, ui) {
                    event.preventDefault();
                    var group = ui.item;
                    window.location.href = '/groups/group-infos.html?id=' + group.id;
                });

            },
            onHidden: function () {
                $searchBarContainer.detach();
                $searchBarContainer.find('input').val("");
                $('.search-bar-wrapper').append($searchBarContainer);
            }
        });
    },
    groupNotFound: function (groupName) {
        var $searchResult = $('#search-results');
        var $groupName = $searchResult.find('.segment-create-group .group-name');
        if ($groupName.length > 0) {
            return $groupName.text(groupName);
        }
        var msgNoGroup =
            `<div class="segment-create-group">\
                <h3>
                    Le Filet <b class="group-name">${groupName}</b> n'existe pas <br>
                    Souhaitez-vous le <a class="btn-create-group">créer </a>?
                </h3>
            </div>`;
        $searchResult.append(msgNoGroup);

        $searchResult.find('.btn-create-group').click(function () {
            //need to get input value too
            var groupName = $('.search-modal .search-bar input').val();
            groupName = groupName ? groupName.trim() : '';
            if (groupName.length === 0) {
                return;
            }
            var data = {
                method: 'create',
                name: groupName
            };
            bg.groupsManager.api.groupsData(data, function (res) {
                if (res.error) {
                    $('.ndl-message-error').text(res.message || NDLConst.ERR_GLOBAL);
                    utils.showHideMessage('error');
                }
                var newGroup = res.group;
                window.location.href = '/groups/group-infos.html?id=' + newGroup.id;
            });

        });
    }
};


$(document).ready(function () {
    commons.onPageReady(function(){
        groups.ready();
    });
});