// (uppercase will be stored to local)
/**
 * global keys
 */
var gKeys = {
    loginCookie: 'ndl_login',
    LIST_GUID: 'listGuid',
    KEY_DEFAULT_HOST_SERVER: 'defaultHost', //default user host
    KEY_SERVERS: 'servers'
};
//hold connected user data
var _userData = {};

// current user manager
var userManager = {
    //server url not set, get the default from config.js
    getHostServer: function () {
        var host = localStorage.getItem(gKeys.KEY_DEFAULT_HOST_SERVER);
        if (!host || null === host) {
            // var options = window.defaultConfig.defaultHost;
            host = window.defaultConfig.BASE_URL_EXT;
            userManager.updateDefaultUserHost(host);
        }
        //remove trailling slashes if any
        return host.replace(/\/+$/, "");
    },
    updateDefaultUserHost: function (host) {
        localStorage.setItem(gKeys.KEY_DEFAULT_HOST_SERVER, host);
    },
    addNewGuidUser: function (newGuid) {
        var listGuid = this.getListGuidUser();
        listGuid = listGuid === null ? [] : listGuid;

        listGuid.push(newGuid);
        localStorage.setObject(gKeys.LIST_GUID, listGuid);
    },
    getListGuidUser: function () {
        return localStorage.getObject(gKeys.LIST_GUID);
    },
    /**
     * return current user data
     */
    getUser: function () {
        return _userData;
    },
    _resetUserData: function () {
        _userData = {
            //ref_count to allow send alert frame
            ref_count: 0,
            user: {
                cle: null,
                id: null,
                host: '',
            },
            groups: {

            }
        };
    },
    checkLogInCookie: function () {
        var url = userManager.getHostServer();
        //Not working with Firefox if port is set
        // (port added only for local server test)
        url = url.replace(':3000', '');
        chrome.cookies.get({
            name: gKeys.loginCookie,
            url: url
        }, function (cookies) {
            utils.debug('cookies ==> ', cookies);
            var user = null;
            if (cookies) {
                try {
                    user = JSON.parse(decodeURIComponent(cookies.value));
                    var connectedUser = _userData.user;

                    if (connectedUser) {
                        if (connectedUser.id !== user.id ||
                            connectedUser.cle !== user.cle) {
                            userManager.onLogOut();
                            _userData.user = user;
                            userManager.onLogIn(user);
                        }
                    }
                    _userData.user = user;
                    //to be sure user data is loaded
                    if (!_userData.info_loaded) {
                        userManager.onLogIn(user);
                    }
                    // else {
                    //     userManager.updateListGuidUser(user);
                    // }

                    // userManager.setUserGuid(user);
                    userManager.updateDefaultUserHost(user.host);
                } catch (e) {
                    utils.debug(e);
                    userManager.onLogOut();
                }
            } else {
                //user is disconnected
                userManager.onLogOut();
            }
        });
    },
    login: function () {
        var urlConnexion = window.defaultConfig.extLoginURL +
            '?redirect=extension';
        chrome.tabs.create({
            url: urlConnexion,
            active: true
        });
    },
    onLogIn: function (user) {
        var data = {
            cle: user.cle
        };
        api.loadUserInfo(data, function (res) {
            if (res.error) {
                return utils.debug('ERR_GET_USER_INFOS');
            }
            _userData.title = res.title;
            _userData.has_email = res.has_email;
            _userData.email = res.email;
            _userData.send_invitation_authorized = res.send_invitation_authorized;
            _userData.info_loaded = true;
        });
        //get user fil
    },
    //update current tab if user is in extension page and loggedOut
    // to allow redirect to connection page
    logOut: function () {
        var self = this;
        self.onLogOut();
        chromeAPI.getActiveTab(function (tab) {
            var urlLogout = window.defaultConfig.extLogoutURL +
                '?redirect=extension';
            chrome.tabs.update(tab.id, {
                url: urlLogout,
                active: true
            });
        });
    },
    isLoggedIn: function () {
        if (_userData.user && _userData.user.cle) {
            return true;
        }
        return false;
    },
    //user logout from the site
    onLogOut: function () {
        userManager._resetUserData();
        localStorage.removeItem(gKeys.KEY_DEFAULT_HOST_SERVER);
        tabsManager.reset();
    },
    /**History manager */
    history: {
        keyLocalHistory: 'userHistory',
        add: function (userCle, id_ref) {
            var userData = userManager.getUser();
            var currentUser = userData.user;
            if (userCle === currentUser.cle) {
                return;
            }
            var history = localStorage.getObject(this.keyLocalHistory) || {};
            var refsHistory = history.refs || {};
            //more than 20 entries in history, remove first added

            var refsArray = Object.values(refsHistory);

            if (refsArray.length >= 20) {
                //order by visited asc and delete the old visited
                refsArray.sort(function (d1, d2) {
                    return d1.visitedAt - d2.visitedAt;
                });
                var oldByVisite = refsArray.shift();
                delete refsHistory[oldByVisite.cle];
            }
            //Save in seconds
            var visitedAt = Math.floor((new Date()).getTime() / 1e3);
            refsHistory[userCle] = {
                cle: userCle,
                id_ref: id_ref,
                visitedAt: visitedAt
            };
            // history['cles'] = clesUsersHistory;
            history['refs'] = refsHistory;
            localStorage.setObject(this.keyLocalHistory, history);
        },
        get: function () {
            var history = localStorage.getObject(this.keyLocalHistory);
            if (!history || !history.refs) {
                return null;
            }
            return history;
        },
        del: function (userCle) {
            var history = localStorage.getObject(this.keyLocalHistory);
            if (!history || history.refs.length === 0) {
                return;
            }
            delete history.refs[userCle];
            if (Object.keys(history.refs).length === 0) {
                return localStorage.removeItem(this.keyLocalHistory);
            }
            localStorage.setObject(this.keyLocalHistory, history);
        }
    }
};


/**
 * Api calls
 */
var api = {
    call: function (url, type, data, cb) {
        if (!userManager.isLoggedIn()) {
            //verify
            userManager._resetUserData();
            userManager.checkLogInCookie();
        }
        if (data) {
            //add version
            var manifest = chrome.runtime.getManifest();
            data = Object.assign({}, data);
            data.v = manifest.version;
            data.src = 'ext';
        }

        $.ajax({
            url: url,
            type: type,
            contentType: 'application/json',
            dataType: 'json',
            data: type === 'POST' ? JSON.stringify(data) : data,
            //timeout: 15000, //15s
            complete: function (jqXHR, textStatus) {
                var res = {};
                if (jqXHR.statusText === 'timeout') {
                    res = {
                        error: true,
                        isOnline: true,
                        message: NDLConst.ERR_TIMEOUT_REQUEST
                    };
                    return cb && cb(res);
                }
                if (jqXHR.readyState !== 4) {
                    res = {
                        error: true,
                        isOnline: true,
                        message: ''
                    };
                    //check if it is internet error
                    api.confirmOnline(function (isOnline) {
                        res.isOnline = isOnline;
                        res.message = isOnline ? NDLConst.ERR_INTERNET_SERVER_UNREACHABLE :
                            NDLConst.ERR_INTERNET_DISCONNECTED;
                        cb && cb(res);
                    });
                    return;
                }
                if (jqXHR.readyState === 4 && jqXHR.status === 404) {
                    res = {
                        error: true,
                        message: 'Not Found'
                    };
                    return cb && cb(res);
                }
                if (jqXHR.readyState === 4 && jqXHR.status === 200) {

                    if (jqXHR.responseJSON) {
                        res = jqXHR.responseJSON;
                    } else {
                        var ctype = jqXHR.getResponseHeader("content-type") || "";
                        if (ctype.indexOf('html') < 0) {
                            //it is not html
                            try {
                                res = JSON.parse(jqXHR.responseText);
                            } catch (e) {
                                utils.debug('ERR_EXTENSION_PARSE_JSON', e.message);
                                res = {
                                    error: true,
                                    code: 'ERR_EXTENSION_PARSE_JSON',
                                    message: NDLConst.ERR_GLOBAL
                                };
                            }
                        } else {
                            res = jqXHR.responseText;
                        }
                    }
                    if (res.isLogged !== undefined && !res.isLogged) {
                        var message = {
                            subject: 'logout'
                        };
                        //send message to popup to prompt for loggin if open
                        messaging.sendInternal(message);
                        userManager.onLogOut();
                    }
                    cb && cb(res);
                }
            }
        });
    },
    confirmOnline: function (cb) {
        var img = document.createElement('img');
        img.src = "https://www.google.com/favicon.ico?_" + (Math.floor(Math.random() * 1e10));
        img.addEventListener('error', onImgError);
        img.addEventListener('load', onImgload);

        var timeout = setTimeout(function () {
            cb(false);
            img.removeEventListener('error', onImgError);
            img.removeEventListener('load', onImgload);
            img.src = ""; //cancel
            img = null;
        }, 7000);

        function onImgload() {
            cb(true);
            clearTimeout(timeout);
        }

        function onImgError() {
            cb(false);
            clearTimeout(timeout);
        }
    },
    buildApiUrl: function (path) {
        var host = userManager.getHostServer();
        return host + '/api/' + path;
    },
    get: function (path, data, cb) {
        data = data || {};
        this.call(this.buildApiUrl(path), 'GET', data, cb);
    },
    post: function (path, data, cb) {
        data = data || {};
        this.call(this.buildApiUrl(path), 'POST', data, cb);
    },
    /**
     * Fetch data from external location(e.g: articles img)
     */
    getExteranl: function (url, data, cb) {
        this.call(url, 'GET', data, cb);
    },
    /**
     * Get articles
     * @param {Object} data the data
     * @property {Number} page page index
     */
    getArticles: function (data, cb) {
        this.get('links/articles', data, cb);
    },
    getLinkInfo: function (data, cb) {
        this.get('links/link_info', data, cb);
    },
    saveLink: function (linkData, cb) {
        this.post('links/add_link', linkData, cb);
    },
    discover: function (data, cb) {
        this.get('links/discover', data, cb);
    },
    /**
     * 
     * @param {Object} data with linkId and new legende
     * @param {function} cb 
     * @returns {undefined}
     */
    updateLegende: function (data, cb) {
        this.post('refs/update_legende', data, cb);
    },
    deleteRef: function (data, cb) {
        this.post('refs/delete', data, cb);
    },
    /**
     * load user infos
     * @param {Object} data req params
     * @property  {String} cle user key 
     */
    loadUserInfo: function (data, cb) {
        this.get('users/user_info', data, cb);
    },
    updateFilTitle: function (data, cb) {
        this.post('users/update_fil_title', data, cb);
    },
    /**
     * load user history data
     */
    loadHistory: function (data, cb) {
        this.post('users/history', data, cb);
    },
    /**
     * send follow, unfollow, invite people
     */
    userActionPost: function (data, cb) {
        this.post('users/user_actions', data, cb);
    },
    // userActionGet: function(data, cb){
    //     this.get('users/user_actions', data, cb);
    // },
    loadFollowed: function (data, cb) {
        this.get('users/list_followed', data, cb);
    },
    /**
     * Get data for a fil (init, next, prev)
     * @param {Object} data request data
     * @param {Function} cb 
     */
    getFilData: function (data, cb) {
        this.get('refs/fil_data', data, cb);
    },
    getFilDates: function (data, cb) {
        this.get('refs/fil_dates', data, cb);
    },
    myFil: function (data, cb) {
        // this.getFilData(data, cb);
        this.getFilData(data, function (res) {
            if (!res.error && !data.filter) {
                var userData = userManager.getUser();
                userData.ref_count = res.ref_count;
            }
            cb(res);
        });
    },
    counters: function (data, cb) {
        var userData = userManager.getUser();
        if (userData.user.cle === data.user_cle) {
            return;
        }
        this.post('refs/counters', data, cb);
    }

};

/**
 * Manager browser action update, tabs data and listener
 */
var tabsManager = {
    //stored fetched tab data, allow also updating popup faster
    tabsData: {},
    reset: function () {
        this.tabsData = {};
    },
    init: function () {
        var bgColor = '#DD582B';
        chrome.runtime.onInstalled.addListener(function (details) {
            chrome.browserAction.setBadgeBackgroundColor({
                color: bgColor
            });
            if (details.reason === "install") {
                var homePage = chrome.runtime.getURL('/home.html');
                chrome.tabs.create({
                    url: homePage
                });
                api.post('needle-installed', {}, function (res) {
                    utils.debug('Install resp: ', res);
                });
            }
        });
        chrome.browserAction.setBadgeBackgroundColor({
            color: bgColor
        });

        //Listener when tab is changed 
        chrome.tabs.onActivated.addListener(function (activeInfo) {
            chromeAPI.getTabDetails(activeInfo.tabId, function (tab) {
                if (!tab || !utils.isSupportedProtocol(tab.url)) {
                    return;
                }
                var tabData = tabsManager.getTabData(tab.id, tab.url);
                if (tabData) {
                    return tabsManager.updateBadge(tab.id, false);
                }
                //do not fire update when loading, il will be done by onCommitted
                if (tab.status === 'loading') {
                    return;
                }
                tabsManager.triggerUpdateBage({
                    id: tab.id,
                    url: tab.url
                });
            });
        });

        var timeoutUpdateBage, urlToFetch;
        var removed = false,
            url = chrome.extension.getURL('/monfil.html');
        chrome.tabs.onUpdated.addListener(function (tabId, changedInfo, tabToRemove) {
            var userHost = userManager.getHostServer();
            userHost = userHost.replace(/^https?:\/\//i, '');
            var tabUrl = tabToRemove.url.replace(/^https?:\/\//i, ''),
                afterSignUrl = userHost += '/ndl_ext_signed_in#SIGNED';

            if (tabUrl === afterSignUrl) {
                if (removed) {
                    return;
                }
                //set it true here to block next loading
                removed = true;
                chromeAPI.getTabIndex(tabToRemove.id, function (index) {
                    var queryInfo = {
                        url: url,
                        active: true,
                        index: index
                    };
                    chrome.tabs.create(queryInfo, function (tab) {
                        chrome.tabs.remove(tabToRemove.id);
                        // removed = false;
                    });
                });
            }
            if (changedInfo.status === 'loading') {
                //ensure logged
                userManager.checkLogInCookie();
            }
            if (changedInfo.status === 'complete') {
                removed = false;
            }
            //onCommitted for frameId 0 does not fire in websites that uses
            //iframe to update page content which then update browser url, so 
            //this do the trick to update badge text accordingly
            if (changedInfo.status === 'complete' && (tabToRemove.url !== urlToFetch)) {
                urlToFetch = tabToRemove.url;
                tabsManager.triggerUpdateBage({
                    id: tabToRemove.id,
                    url: urlToFetch
                });
            }
        });

        //listen to these two listener to prevent fetching data for each url
        //when redirection happens in a tab
        chrome.webNavigation.onBeforeNavigate.addListener(function (details) {
            if (details.frameId === 0) {
                clearTimeout(timeoutUpdateBage);
            }
        });

        chrome.webNavigation.onCommitted.addListener(function (details) {
            if (details.frameId === 0) {
                if (!utils.isSupportedProtocol(details.url)) {
                    return;
                }
                urlToFetch = details.url;
                clearTimeout(timeoutUpdateBage);
                timeoutUpdateBage = setTimeout(function () {
                    tabsManager.triggerUpdateBage({
                        id: details.tabId,
                        url: details.url
                    });
                }, 1500);
            }
        });

        chrome.tabs.onRemoved.addListener(function (tabId, removedInfo) {
            tabsManager.removeTabData(tabId);
        });
    },
    /**
     * Fetch data to update popup icon and counter
     */
    triggerUpdateBage: function (tabData) {
        var url = tabData.url,
            tabId = tabData.id;
        if (!utils.isSupportedProtocol(url)) {
            return;
        }
        var data = {
            url: url
        };
        api.getLinkInfo(data, function (res) {
            if (!res.error) {
                //check if tab is not closed and has alwas the same url
                // before storing any data
                chromeAPI.getTabDetails(tabId, function (tab) {
                    if (tab && (tab.url === url)) {
                        tabsManager.tabsData[tabId] = res.data;
                        tabsManager.updateBadge(tabId, false);
                    }
                });
            } else {
                //must be delete as it can be a different link
                tabsManager.removeTabData(tabId);
                tabsManager.updateBadge(tabId, res.error);
            }
        });
    },
    /**
     * Get fetched data for a tab, url required to validate 
     * whelter the current url of the tab is always the same 
     * as the fetched url
     * @param {Number} tabId 
     * @param {String} url
     */
    getTabData: function (tabId, url) {
        if (!url || !this.tabsData[tabId] || !this.tabsData[tabId].url) {
            return null;
        }
        var urlTabsData = this.tabsData[tabId].url;
        urlTabsData = utils.normalizeUrl(urlTabsData);
        url = utils.normalizeUrl(url);

        return urlTabsData === url ?
            this.tabsData[tabId] : null;
    },
    setTabData: function (tabId, data) {
        if(Object.keys(data).length === 0){
            return delete this.tabsData[tabId];
        }
        var tabData = this.tabsData[tabId] || {};
        //update or create property (mostly for legende)
        tabData = Object.assign(tabData, data);
        this.tabsData[tabId] = tabData;
    },
    updateTabData: function (newData) {
        var idRef = newData.id,
            tabsData = this.tabsData,
            values = Object.values(tabsData);

        if (values.length === 0) {
            return;
        }
        //in case fil is open in multiple tabs or windows
        for (var i = 0, tabData; i < values.length; i++) {
            tabData = values[i];
            if (tabData.id === idRef) {
                Object.assign(tabData, newData);
            }
        }
    },
    /**
     * remove data for tabId from bg en storage.local
     */
    removeTabData: function (tabId) {
        delete this.tabsData[tabId];
        chrome.storage.local.remove(tabId.toString());
        this.removeActiveCsFil(tabId);
    },
    setBadgeInfos: function (tabId, badgeText, iconPath) {
        chromeAPI.getTabDetails(tabId, function (tab) {
            if (tab) {
                chrome.browserAction.setIcon({
                    path: iconPath,
                    tabId: tabId
                });
                chrome.browserAction.setBadgeText({
                    text: ((typeof badgeText === 'undefined' || badgeText <= 0) ? '' : badgeText + ''),
                    tabId: tabId
                });
            }
        });
    },
    /**
     * Update browser action text and icon
     * @param {Number} tabId the id of the tab to update
     * @param {boolean} error indicate the reason of update
     */
    updateBadge: function (tabId, error) {
        var iconNoFil = 'img/icons/needle-no-fil-128.png';
        var iconWithFil = 'img/icons/needle-fil-128.png';
        if (error) {
            return this.setBadgeInfos(tabId, '', iconNoFil);
        }
        var dataTab = this.tabsData[tabId];
        if (!dataTab) {
            return;
        }
        var badgeText = dataTab.local_ref;
        var icon = iconNoFil;
        if (dataTab.referenced) {
            badgeText--;//-1 as current user has referenced it
            icon = iconWithFil;
        }
        this.setBadgeInfos(tabId, badgeText, icon);
    },
    /**
     * Hold active fil in content script,object like
     * tabId: {userCle: userCle, linkId: id, activeLinkId: id,
     * fil: {refs: r, dates: d}}
     * if activeLinkId is different to linkId, then data for the fil
     * must be requested using activeLinkId. by default activeLinkId = linkId  
     */
    _activeCsFils: {},
    /**
     * Return active fil in tabId
     * @param {Number} tabId tab id of the fil
     */
    getActiveCsFil: function (tabId) {
        return this._activeCsFils[tabId] || null;
    },
    /**
     * Add a user fil as active in content script
     * @param {Number} tabId id of the tab
     * @param {Object} ref active ref
     * @property {String} userCle user cle of this fil
     * @property {Number} linkId active linkid of this fil
     * @property {Number} idRef active ref id
     */
    addActiveCsFil: function (tabId, ref) {
        //avoid FF dead obj
        ref = Object.assign({}, ref);
        this._activeCsFils[tabId] = {
            ref: ref
            // fil: null
        };
    },
    /**
     * 
     * @param {Number} tabId id of the tab
     * @param {String} attrKey the key to update
     * @param {any} value key value
     */
    setActiveCsFilParam: function (tabId, attrKey, value) {
        var activeFil = this.getActiveCsFil(tabId);
        if (!activeFil) {
            return;
        }
        if (attrKey === 'ref') {
            value = Object.assign({}, value);
        }
        activeFil[attrKey] = value;
    },
    /**
     * remove this tab data (when tab is closed for ex.)
     */
    removeActiveCsFil: function (tabId) {
        delete this._activeCsFils[tabId];
    },
    /**
     * Notify all tabs containing a fil data was changed, and update
     * it contain if it's concerned
     */
    notifyCsFilsDataChanged: function(what, data){
        var tabIds = Object.keys(this._activeCsFils);
        var message = {
            subject: 'notify_data_changed',
            what: what,
            body: data
        };
        
        for (var i = 0, tabId; i < tabIds.length; i++) {
            tabId = +tabIds[i];
            messaging.sendMessage(tabId, message, function(res){
                utils.debug('notify Cs fil data changed ==> ', res);
            }); 
        }
    }
};


/**
 * extension messaging
 */
var messaging = {
    //send in the extension
    sendInternal: function (message, cb) {
        chrome.runtime.sendMessage(message, function (response) {
            //need to check lastError in FF
            utils.debug('lastError internal ==> ', chrome.runtime.lastError);
            // utils.debug('Response to <' + message.subject + '> : ', response);
        });
    },
    //send to content script
    sendMessage: function (tabId, message, cb) {
        chrome.tabs.sendMessage(tabId, message, function (response) {
            //need to check lastError in FF
            utils.debug('lastError external ==> ', chrome.runtime.lastError,
                ' tabid ==> ', tabId, ' message => ', message);
            // utils.debug('Response to <' + message.subject + '> : ', response);
            cb && cb(response);
        });
    },
    /**
     * send message to all tabs
     */
    broadcast: function (message, cb) {
        var self = this,
            regExcludeChromeExtension = /^chrome-extension:\/\/*/,
            regExcludeMozExtension = /^moz-extension:\/\/*/;
        chrome.tabs.query({}, function (tabs) {
            for (var i = 0, tab; i < tabs.length; ++i) {
                tab = tabs[i];
                if (!regExcludeChromeExtension.test(tab.url) &&
                    !regExcludeMozExtension.test(tab.url) ) {
                    self.sendMessage(tab.id, message);
                }
            }
        });
    },
    onMessage: function () {
        chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {
            var from = sender.tab ? (sender.tab.id + ': ' + sender.tab.url) : 'the extension';
            utils.debug('Incomming message from : ', from);
            var subject = request.subject;
            if (subject && processIncomingMessages[subject]) {
                processIncomingMessages[subject](request, sender, sendResponse);
                return true;
            }
        });
    }
};

/**
 * process received messages
 */
var processIncomingMessages = {
    /**
     * initialize CS fil
     */
    init_frame: function (req, sender, sendResp) {
        var subject = req.subject;
        var senderId = sender.tab.id;
        var tabInfo = tabsManager.getActiveCsFil(senderId);

        utils.debug(tabInfo);
        if (!tabInfo) {
            //no fil is active in this tab
            utils.debug('No active fil in tab', senderId);
            return;
        }
        var linkId = tabInfo.ref.linkId,
            // fil = tabInfo.fil,

            body = {
                tabId: senderId,
                currentUser: userManager.getUser(),
                // fil: null,
                isLogged: userManager.isLoggedIn()
            };

        if (!body.isLogged) {
            return sendResp({
                subject: subject,
                body: body
            });
        }

        var data = {
            method: 'init',
            user_cle: tabInfo.ref.userCle,
            link_id: linkId,
            content_request: true
        };

        api.getFilData(data, function (res) {
            utils.debug(res);
            if (res.error) {
                utils.debug('ERR_INIT_CONTENT_REQ');
                return;
            }
            var idxActiveRef = utils.findIndexRef(res.refs, 'link_id', data.link_id);
            body.fil = res;
            body.fil.activeRef = {
                userCle: data.user_cle,
                link_id: data.link_id,
                idRef: res.refs[idxActiveRef].id
            };
            sendResp({
                subject: subject,
                body: body
            });
        });
    },
    /**
     * send more refs to CS 
     */
    load_more_refs: function (req, sender, sendResp) {
        var subject = req.subject;
        var body = req.body;

        api.getFilData(body, function (res) {
            if (res.error) {
                utils.debug('ERR_LOAD_MORE: method => ', res.error, body.method);
            }
            sendResp({
                subject: subject,
                body: res
            });
        });
    },
    /**
     * A ref on CS fil is clicked
     */
    browse_content_fil: function (req, sender, sendResp) {
        var subject = req.subject;
        var senderId = sender.tab.id;
        var data = req.body;

        var tabInfo = tabsManager.getActiveCsFil(senderId);
        if (!tabInfo) {
            return utils.debug('Something got wrong, with ',
                senderId, ' page reload needed');
        }
        //update only active link, init_frame will send updated data
        //after navigation to clicked ref
        tabInfo.ref.linkId = data.link_id;
        tabInfo.ref.idRef = data.id_ref;
        sendResp({
            subject: subject,
            body: data
        });
        //save current ref in history (history is saved only on
        // content script navigation)
        userManager.history.add(data.user_cle, data.id_ref);
        //update counter clicks 
        var dataCounter = {
            id_ref: data.id_ref,
            user_cle: data.user_cle,
            counter: 'clicks'
        };

        api.counters(dataCounter);
    },
    load_fil_dates: function (req, sender, sendResp) {
        var subject = req.subject;
        var data = req.body;
        api.getFilDates(data, function (res) {
            utils.debug(res);
            if (res.error) {
                return utils.debug('ERR_LOAD_FIL_DATES');
            }
            sendResp({
                subject: subject,
                body: res
            });
        });
    },
    slider_handle_drag: function (req, sender, sendResp) {
        var subject = req.subject;
        var data = req.body;
        data.method = 'slider';
        data.content_request = true;
        api.getFilData(data, function (res) {
            //do not update CS fil (update only when browse to another page)
            sendResp({
                subject: subject,
                body: res
            });
        });
    },
    follow_unfollow: function (req, sender, sendResp) {
        var subject = req.subject,
            // senderId = sender.tab.id,
            data = req.body;
        api.userActionPost(data, function (res) {
            
            // var tabInfo = tabsManager.getActiveCsFil(senderId);
            // if (!res.error && tabInfo && tabInfo.fil) {
            //     tabInfo.fil.is_following = res.is_following;
            // }
            sendResp({
                subject: subject,
                body: res
            });
        });
    },
    contact_request: function (req, sender, sendResp) {
        var subject = req.subject;
        var data = req.body;
        groupsManager.contactRequest(data.user);
        sendResp({
            subject: subject,
            body: null
        });
    },
    /**
     * If notified by Cs that ref is deleted, send message to
     * active CS fils to delete this ref from fil
     */
    on_ref_deleted: function (req, sender, sendResp) {
        var body = req.body,
            idRefToDel = body.id_ref,
            activeCsFils = tabsManager._activeCsFils || {},
            tabIds = Object.keys(activeCsFils);

        if (tabIds.length === 0) {
            return;
        }
        var message = {
            subject: 'remove_deleted_ref_slide',
            body: {
                id_ref: idRefToDel
            }
        };
        // var tabData = tabsManager.tabsData;
        for (var i = 0, tabId, dataCsFil; i < tabIds.length; i++) {
            tabId = +tabIds[i];
            dataCsFil = activeCsFils[tabId];
            //delete tabData for popup badge            
            delete tabsManager.tabsData[tabId];
            
            messaging.sendMessage(tabId, message, function (response) {
                if (!response) {
                    utils.debug('Delete ref - No response');
                    return;
                }
                body = response.body;
                var newActiveRef = body.ref;
                if (null === newActiveRef) {
                    return;
                }
                dataCsFil.ref.idRef = newActiveRef.idRef;
                dataCsFil.ref.linkId = newActiveRef.linkId;

            });
        }
    },
    /**
     * send local history fil
     */
    load_history: function(req, sender, sendResp) {
        var subject = req.subject,
            history = localStorage.getObject(userManager.history.keyLocalHistory) || {};
        sendResp({
            subject: subject,
            body: history
        });
    }
};

/**
 * Manager groups and contact users 
 */
var groupsManager = {
    //api for groups
    api: {
        contactUser: function (messageData, cb) {
            api.post('contact-user', messageData, cb);
        },
        /**
         * fetch group data
         * @param {Object} data param request
         * @property {String} method indicate the action to perform
         */
        groupsData: function (data, cb) {
            api.get('groups/group_data', data, cb);
        },
        bugRepport: function (messageData, cb) {
            var data = {
                message: messageData
            };
            api.post('bug-report', data, cb);
        },

    },
    userToContact: null,
    getUserToContact: function () {
        return this.userToContact;
    },
    contactRequest: function (user) {
        this.userToContact = user;
        var path = '/groups/contact-user.html';
        chromeAPI.isExtensionUrlOpen(path, function (tab) {
            if (tab) {
                chrome.tabs.update(tab.id, {
                    url: chrome.runtime.getURL(path),
                    active: true
                });
            } else {
                chrome.tabs.create({
                    url: chrome.runtime.getURL(path),
                    active: true
                });
            }
        });
    }
};


//init all
userManager.checkLogInCookie();
tabsManager.init();
messaging.onMessage();




//==========Side menu html ==========
function getSideMenu() {
    var userData = _userData;
    var user = userData.user;
    var menuItems;
    var userIdOrEmail = "";
    var urlLogin = window.defaultConfig.extLoginURL + '?redirect=extension';
    var urlLogout = window.defaultConfig.extLogoutURL + '?redirect=extension';
    if (null !== user && user.id) {
        userIdOrEmail = userData.email? userData.email : user.id;
        menuItems =
            `<a href=" ${urlLogout}" class="menu-item disconnect">
                <span class="mobile-item"><i class="la la-power-off"></i>Déconnexion</span>
                <span class="wide-item">Déconnexion</span>
            </a>`;

        if(userData.send_invitation_authorized){
            menuItems +=   
                `<a class="menu-item invitations">
                    <span class="mobile-item"><i class="la la-user-plus"></i>Inviter une personne</span>
                    <span class="wide-item">Inviter une personne</span>
                </a>`;
        }
    } else {
        menuItems =
            `<a href="${urlLogin}" class="menu-item">
                <span class="mobile-item"><i class="la la-sign-in"></i>Connexion</span>
                <span class="wide-item">Connexion</span>
            </a>`;
    }

    var sideMenu =
        `<section class="ndl-side-menu right">
            <div class="ndl-close-side-menu-holder">
                <div class="ndl-close-side-menu-holder-inner">
                    <a class="ndl-close-side-menu"><span class="la la-close"></span></a>
                </div>
            </div>
            <div class="widget">
                <a class="ndl-user-img-widget">
                    <div class="ndl-user-img">
                        <span class="la la-user ndl-side-menu-user-icon"></span>
                    </div>
                    <div class="ndl-user-id">${userIdOrEmail}</div>
                </a>
            </div>
            <div class="widget widget_nav_menu">
                <div><div class="menu">${menuItems}</div></div>
            </div>
        </section>`;

    return sideMenu;
}