var bg = chrome.extension.getBackgroundPage();
var commons = {
    /**
     * Make easy to handle privacy agrement on extension
     * page ready
     * @param {function} fn function to execute
     */
    onPageReady: function (fn) {
        var ndlPrivacy = this.ndlPrivacy;
        ndlPrivacy.privacyConsentCheck(function (res) {
            if (!res) {
                ndlPrivacy.showPrivacyModal(fn);
                return;
            }
            fn();
        });
    },
    /**
     * Handle privacy agrement
     */
    ndlPrivacy: {
        NDL_CONSENT: 'privacyconsent',
        privacyConsentCheck: function (cb) {
            var self = this;
            chrome.storage.local.get(self.NDL_CONSENT, function (result) {
                cb(result[self.NDL_CONSENT]);
            });
        },
        consentToPrivacy: function (cb) {
            var data = {};
            data[this.NDL_CONSENT] = true;
            chrome.storage.local.set(data, function () {
                chrome.runtime.lastError ? cb(false) : cb(true);
            });
        },
        showPrivacyModal: function (cb) {
            var self = this,
                $body = $('body'),
                $pageContent = $('.page-content'),
                $privacyContainer = $('<div class="ndl-privacy-consent-container">');
            $pageContent.addClass('loading');
            $body.append($privacyContainer).addClass('overflow');
            $privacyContainer.load("/privacy.html", function () {
                $privacyContainer.addClass('active');
                $body.removeClass('overflow');
                $privacyContainer.find('.btn-accept').click(function () {
                    self.consentToPrivacy(function (res) {
                        if (!res) {
                            //handle error
                            return;
                        }
                        $privacyContainer.remove();
                        cb();
                    });
                });
                setTimeout(() => {
                    $pageContent.removeClass('loading');
                }, 200);
            });
        },
    },
    template: {
        popupEditLegende: function () {
            var t = `<div class="popup-fil popup-edit-legende"> 
                <div class="popup-content-wrapper">
                    <div class="popup-body">
                        <div class="content">
                            <h3 class="header">
                                <span>éditer la légende</span>
                                <button class="close">
                                    <i class="la la-close"></i>
                                    <span class="esc">esc</span>
                                </button>
                            </h3>
                            <div class="form">
                                <div class="field">
                                    <textarea placeholder="Votre légende peut comporter des #hashtags"></textarea>
                                </div>
                                <div class="ndl-message ndl-message-error"></div>
                                <div class="ndl-message ndl-message-success"></div>
                            </div>
                        </div>
                        <div class="actions">
                            <div class="ui button close-popup-btn">Annuler</div>
                            <div class="ui button btn-validate btn-validate-edit-title">éditer</div>
                        </div>
                    </div>
                </div></div>`;
            return $(t);
        },
        popupDeleteRef: function () {
            var t = `<div class="popup-fil popup-delete-ref">
                <div class="popup-content-wrapper">
                    <div class="popup-body">
                        <div class="content">
                            <h3 class="header">
                                <span>Supprimer la référence </span>
                                <button class="close">
                                    <i class="la la-close"></i>
                                    <span class="esc">esc</span>
                                </button>
                            </h3>
                            <div class="content-inner">
                                <div class="message-content"></div>
                                <div class="ui indeterminate text loader">Envoi en cours ...</div>
                            </div>
                        </div>
                        <div class="actions">
                            <div class="ui button close-popup-btn">Fermer</div>
                            <div class="ui button btn-send-mail">Envoyer</div>
                        </div>
                    </div>
                </div></div>`;
            return $(t);
        },
        dialogNotifyUpdateTitle: function () {
            var t = `<div class="dialog-fil dialog-notify-update-title">
                <div class="dialog-content-wrapper">
                    <div class="dialog-body">
                        <div class="content">
                            <button class="close">
                                <i class="la la-close"></i>
                                <span class="esc">esc</span>
                            </button>
                            <h3 class="header">
                                <span class="icon">Personnaliser le titre de votre fil</span>
                            </h3>
                            <div class="content-body">
                                N'oubliez pas de donner un titre à votre fil afin d'aider les autres utilisateurs à l'identifier.
                            </div>
                        </div>
                    </div>
                </div>
            </div>`;
            return $(t);
        }
    },
    /**
     * Show global page message
     * @param {String} message the message to show
     */
    showPageMessage: function (message, type) {
        var $pageMsg = $('.page-global-msg');
        var $content = $pageMsg.find('.content');
        if (type === 'info') {
            $content.html('<h2 class="info-loading-page">' + message + '</h2>');
        } else {
            $content.html('<h2 class="error-loading-page">' + message + '</h2>');
        }
        $pageMsg.fadeIn(1300);
    },
    /**
     * placeholder data loading
     */
    contentPlaceholder: function () {
        var $template = $(this.templateSlide());

        $template.find('.date, .ref-count, .title, .legende, .domain').empty();
        $template.find('.date, .ref-count, .domain').addClass('holder');
        var holder, str;
        holder = str = '<div class="holder"></div>';
        for (var i = 0; i < 3; i++) {
            str += holder;
        }
        $template.find('.legende').html(str);

        str += holder;
        $template.find('.title').html(str);

        $template.find('.slide-content').addClass('placeholder');
        $template.addClass('placeholder-slide');

        $template.find('.ref-actions').html(holder + holder);
        return $template;
    },
    templateSlide: function (templateData) {
        var data = templateData ? templateData : {};
        var legende = data.legende ? utils.autoLinkHashtags(data.legende) : data.legende,
            createdAt = data.createdAt ? utils.slideShortDate(data.createdAt) : data.created_at,
            localRef = data.localRef,
            title = data.title,
            domain = data.domain,
            titleDate = utils.toISOString(data.createdAt);
        var t = `<li class="swiper-slide"><div class="slide-inner">
            <div class="slide-content">
                <div class="layout top-layout">
                    <div class="header">
                        <div class="date" title=${titleDate}>${createdAt}</div>
                        <div class="ref-count">${localRef}</div>
                    </div>
                    <div class="footer">
                        <div class="link-title"><span class="title">${title}</span></div>
                    </div>
                    <div class="domain-container">
                        <a class="domain"><i class="la la-link"></i> ${domain}</a>
                    </div>
                </div>
                <div class="separator"></div>
                <div class=" layout bottom-layout">
                    <div class="header clamp">
                        <div class="legende"><div class="content">${legende}</div></div>
                    </div>
                    <div class="footer ref-actions">
                        <span class="action edit-legende" data-title="&Eacute;diter la légende">
                            <i class="la la-pencil"></i>
                        </span>
                        <span class="action del-ref" data-title="Supprimer cette référence">
                            <i class="la la-trash"></i>
                        </span>
                        <span class="action memorise-fil-ref" data-title="Mémoriser ce fil depuis cette référence">
                            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 100 100">
                                <path d="M78.67 95V11.17A6.16 6.16 0 0 0 72.51 5h-45a6.16 6.16 0 0 0-6.16 6.16V95L50 80z"/>
                            </svg>
                        </span>
                    </div>
                </div>
            </div>
        </div></li>`;
        return $(t);
    },
    createSwiper: function (cleUser) {

        var $fil = $('.fil-' + cleUser);
        if ($fil.length === 0) {
            return null;
        }
        var swiperSelector = '.fil-' + cleUser + ' .swiper-container';
        var self = this;
        var swiper = new Swiper(swiperSelector, {
            init: false,
            cleUser: cleUser,
            slidesPerView: 'auto',
            spaceBetween: 45,
            centeredSlides: true,
            slideToClickedSlide: true,
            // simulateTouch: false,
            keyboard: {
                enabled: true,
            },
            on: {
                slideChangeTransitionStart: function () {
                    self.loadMoreRefs(this);
                    if (this.slides.length > 2) {
                        $('.swiper-button').addClass('ndl-active');
                    }

                }
            }
        });
        //swiper is not init if it has not prop slides
        return swiper.slides ? swiper : null;
    },
    /**
     * return swiper instance 
     */
    getSwiperInstance: function () {
        var $swiperContainer = $('.fil .swiper-container');
        if ($swiperContainer.length === 0) {
            return null;
        }
        var swiper = $swiperContainer[0].swiper;
        return swiper.slides ? swiper : null;
    },
    /**
     * Set appropriate fil color class according color priorities 
     * and remove other color classes
     */
    setFilColor: function (colorsPriorities, userCle) {
        var colorData = utils.getFilColor(colorsPriorities, userCle);
        if (colorData.color) {
            var $fil = $('.fil'),
                classes = utils.filColorsClasses;
            $fil.removeClass(classes);
            $fil.addClass(colorData.color);
        }
    },
    /**
     * Suggest user to personalize his fil title after adding a ref
     * if he has less than 5 refs and the title is not
     * personalized
     */
    suggestUpdateFilTitle: function () {
        var $fil = $('body.monfil .fil'),
            $filTitle = $('.fil-title');
        if ($fil.length === 0 || $filTitle.length === 0) {
            return;
        }
        var title = $filTitle.find('.title-text').text().trim();
        if (title.length === 0) {
            return;
        }
        var defaultTitle = 'Fil sans titre';
        if (defaultTitle.toLocaleLowerCase() !== title.toLocaleLowerCase()) {
            return;
        }
        $('.dialog-notify-update-title').remove();
        var $dialogNotifyUpdateTitle = this.template.dialogNotifyUpdateTitle();
        $fil.append($dialogNotifyUpdateTitle);
        $dialogNotifyUpdateTitle.addClass('active');
        
        $dialogNotifyUpdateTitle.find('.close')
            .click(function () {
                $dialogNotifyUpdateTitle.remove();
            });

        $(document).off('keyup').keyup(function (e) {
            if (e.keyCode === 27) {
                $dialogNotifyUpdateTitle.remove();
            }
        });

    },
    /**
     * Initialze swiper with fil data
     * @param {Object} filData data of the fil
     */
    initializeSwiper: function (filData, initialSlide, swiper) {
        var cleUser = filData.user.cle;
        if (null === swiper) {
            utils.debug('ERR_INITIALIZE_SWIPER');
            return null;
        }
        var refs = filData.refs,
            filSelector = '.fil-' + cleUser;

        swiper.params.cleUser = cleUser;
        swiper.params.user = filData.user;
        swiper.params.filParams = {
            firstRefLinkId: filData.first_ref_link_id,
            lastRefLinkId: filData.last_ref_link_id,
            refCount: filData.ref_count
        };
        var $ul = $('<ul></ul>');
        for (var i = 0, $slide; i < refs.length; i++) {
            $slide = this.createSlide(refs[i]);
            //prepend to create child in good order
            $ul.prepend($slide);
        }
        var slides = $ul.children(),
            direction = 'init';
        this.addSlides(slides.toArray(), swiper, direction);
        // swiper.prependSlide($ul.children());
        var $fil = $(filSelector);
        if (filData.ref_count > 2) {
            // show navigation btn
            $fil.find('.swiper-button').addClass('ndl-active');

            $fil.find('.swiper-button-prev').off('click').click(function () {
                swiper.slidePrev();
            });

            $fil.find('.swiper-button-next').off('click').click(function () {
                swiper.slideNext();
            });
        }
        sliderBar.destroy();
        if (filData.ref_count > 10) {
            var urlParams = utils.urlParams(window.location.href);
            var data = {
                user_cle: cleUser,
                filter: urlParams.filter,
                term: urlParams.term,
                selected_tags: urlParams.selected_tags
            };
            var id_ref = refs[initialSlide].id;
            this.initFilDates(data, id_ref, swiper);
        }

        swiper.params.initialSlide = initialSlide;

        //init now swiper to bind events and validate initialSlide
        swiper.init();
        swiper.slideTo(initialSlide, 0);

        if (filData.followedRef && filData.followedRef.id) {
            this.setMemorizeRefSate(true, filData.followedRef);
        } else {
            //to clear data memorized ref in data fil too
            this.setMemorizeRefSate();
        }
        this.setFilColor(filData.fil_colors, cleUser);
        return true; //swiper is init
    },
    /**
     * Create a swiper slide and add events
     */
    createSlide: function (ref) {

        var userData = bg.userManager.getUser(),
            user = userData.user;
        if (!user.cle) {
            utils.debug('Something got wrong! no user');
            return;
        }

        //minus 1 for visible ref
        var localRef = ref.link.local_ref - 1,
            isCurrentUser = ref.user_cle === user.cle;
        var dataTemplate = {
            legende: ref.legende,
            createdAt: ref.created_at,
            localRef: localRef,
            title: ref.link.title,
            domain: utils.hostname(ref.link.url),
        };
        var $slide = this.templateSlide(dataTemplate);
        var $slideContent = $slide.find('.slide-content');
        $slide.attr({
            'data-id': ref.id,
            'data-link-id': ref.link_id,
            'data-url': ref.link.url
        });

        // =========legende========
        if (null === ref.legende || ref.legende.trim().length === 0) {
            var $legende = $slide.find('.legende');
            if (isCurrentUser) {
                var placeholderLegende = `<span class="placeholder-legende">Qu'est-ce que cette page vous inspire ?</span>`;
                $legende.html(placeholderLegende);
            } else {
                $legende.remove();
            }
        }
        var self = this;
        // ========counter===========
        if (localRef <= 0) {
            $slide.find('.ref-count').remove();
        }
        var userCross = ref.crossing;
        if (userCross) {
            $slideContent.addClass('cross');
        }
        if (isCurrentUser) {
            $slideContent.find('.memorise-fil-ref').remove();
            var $fil = $('.fil');

            if ($fil.data('filter') === 'cross') {
                var strDate = 'Dernier croisement : ' + utils.slideShortDate(dataTemplate.createdAt);
                $slideContent.find('.header .date').text(strDate);
            }
        } else {
            $slideContent.find('.edit-legende, .del-ref').remove();
        }
        if (ref.memorized) {
            $('.fil').addClass('following');
            $slideContent.addClass('memorized')
                .find('.memorise-fil-ref')
                .attr('data-title', NDLConst.TITLE_REF_MEMORIZED);
        }

        $slideContent.click(function (e) {
            var action = "browse",
                urlParams = utils.urlParams(window.location.href),
                params = {};
            if (userCross && isCurrentUser) {
                action = "discover";
                if (urlParams.filter === 'cross') {
                    params = {
                        sortBy: 'created_at-desc'
                    };
                }

            }
            self.browseOrDiscoverFil(ref, action, params);
        });

        var classes = '.domain, .edit-legende, .placeholder-legende, ' +
            ' .del-ref, .memorise-fil-ref, .ndl-hashtag';
        $slideContent.find(classes).click(function (e) {
            e.stopPropagation();
            var $this = $(this);
            if ($this.hasClass('domain')) {
                return self.browseOrDiscoverFil(ref, 'browse');
            }
            if ($this.hasClass('edit-legende') || $this.hasClass('placeholder-legende')) {
                return self.updateRefLegende(ref, $slideContent);
            }
            if ($this.hasClass('del-ref')) {
                return self.deleteRef(ref);
            }
            if ($this.hasClass('memorise-fil-ref')) {
                var isMemorized = $slideContent.hasClass('memorized');
                var dataRef = {
                    id: ref.id,
                    link_id: ref.link_id,
                    user_cle: ref.user_cle
                };
                self.memoriseRef(isMemorized, dataRef);
            }
            if ($this.hasClass('ndl-hashtag')) {
                var hastag = $this.text();
                self.search.triggerHashtagsFilter(hastag);
            }
        });
        return $slide;
    },
    /**
     * Add slides to swiper and replace the loader if any
     * @param {String} direction prev or next 
     * @param {Array} arraySlides html elements 
     * @param {Object} swiper 
     */
    addSlides: function (arraySlides, swiper, direction) {
        var func, indexPlaceholder = -1;
        if (direction === 'init') {
            swiper.prependSlide(arraySlides);
        } else if (direction === 'prev') {
            indexPlaceholder = 0;
            func = 'prependSlide';
        } else if (direction === 'next') {
            indexPlaceholder = swiper.slides.length - 1;
            func = 'appendSlide';
        } else {
            swiper.removeAllSlides();
            swiper.appendSlide(arraySlides);
            swiper.update();
        }
        if (indexPlaceholder >= 0) {
            var $placeholder = $(swiper.slides[indexPlaceholder]);
            setTimeout(function () {
                if ($placeholder.hasClass('placeholder-slide')) {
                    var loaderReplacer = arraySlides.shift();
                    $placeholder.replaceWith(loaderReplacer);
                    swiper.update();
                }
                if (arraySlides.length === 0) {
                    return;
                }
                swiper[func](arraySlides);
                swiper.update();
            }, 800);
        }
    },
    /**
     * Check if ref is stil exist and open fil in a tab(browse)
     * or in discover page
     * @param {object} ref
     * @param {string} action browse|discover
     * @param {object} urlParams Optinnal url params for discover
     */
    browseOrDiscoverFil: function (ref, action, urlParams) {
        var $fil = $('.fil');
        if ($fil.hasClass('loading')) {
            return;
        }
        var data = {
            method: 'ref_info',
            id_ref: ref.id,
            user_cle: ref.user_cle
        };
        var self = this;
        bg.api.getFilData(data, function (res) {
            if (res.error) {
                utils.debug(res);
                var message = (res.error === 'REF_NOT_FOUND') ?
                    NDLConst[res.error] :
                    res.message || NDLConst.ERR_GLOBAL;
                $('.ndl-message-error').text(message);
                utils.showHideMessage('error');
                return;
            }
            if (action === 'discover') {
                self.discover(ref.link_id, urlParams);
            } else {
                $fil.addClass('loading');
                self.browseFil(ref, function () {
                    $fil.removeClass('loading');
                });
            }
        });

    },
    /**
     * Open fil in content script from this ref
     * @param {Object} ref 
     */
    browseFil: function (ref, callback) {
        var userCle = ref.user_cle;
        var link = ref.link;
        async.waterfall([
            function (cb) {
                //search for a tab where this fil maybe active
                chromeAPI.getActiveWinTabs(function (tabs) {
                    for (var i = 0, tab, tabInfo; i < tabs.length; i++) {
                        tab = tabs[i];
                        tabInfo = bg.tabsManager.getActiveCsFil(tab.id);
                        if (tabInfo && tabInfo.ref.userCle === userCle) {
                            return cb(null, tab.id);
                        }
                    }
                    cb(null, null);
                });
            },
            function (tabId, cb) {
                if (!tabId) {
                    return cb(null, null);
                }
                chromeAPI.getTabDetails(tabId, function (tab) {
                    cb(null, tab);
                });
            },
            function (tab, cb) {
                if (null !== tab) {
                    return cb(null, false, tab);
                }
                chromeAPI.createTab(link.url, function (tab) {
                    cb(null, true, tab);
                });

            },
            function (created, tab) {
                //neede id ref for slider
                var refData = {
                    userCle: userCle,
                    linkId: ref.link_id,
                    idRef: ref.id
                };
                if (created) {
                    bg.tabsManager.addActiveCsFil(tab.id, refData);
                } else {
                    //for update, remove the key from storage so fil will be automatically opened
                    var key = tab.id.toString();
                    chrome.storage.local.remove(key, function () {
                        bg.tabsManager.setActiveCsFilParam(tab.id, 'ref', refData);
                        chrome.tabs.update(tab.id, {
                            url: link.url,
                            active: true
                        });
                    });
                }
                callback && callback();
            }
        ]);
    },
    /**
     * Open discover page
     * @param {number} linkId link id to discover
     * @param {object} params optionnal params to add to discover url
     */
    discover: function (linkId, params) {
        params = params || {};

        var path = '/discover.html';
        var urlParams = {
            link_id: linkId,
            sort_by: params.sortBy || 'created_at-asc'
        };
        linkId = linkId.toString();
        var urlDiscoverPage = chrome.runtime.getURL(
            '/discover.html?' + $.param(urlParams));
        chromeAPI.isExtensionUrlOpen(path, function (tabDiscover) {
            if (tabDiscover) {
                var url = tabDiscover.url;
                var parsedUri = utils.parseUri(url);
                var currentDiscoverLinkId = parsedUri.queryKey.link_id;
                var update = {
                    active: true,
                };
                if (currentDiscoverLinkId !== linkId) {
                    update.url = urlDiscoverPage;
                }
                chrome.tabs.update(tabDiscover.id, update);
            } else {
                chrome.tabs.create({
                    url: urlDiscoverPage,
                    active: true
                });
            }
        });
    },
    viewFil: function (ref) {
        var path = '/viewfil.html',
            urlParams = {
                cle: ref.user_cle,
                link_id: ref.link_id,
                ref: ref.link_id //ref is link_id of active ref
            },
            urlViewFil = chrome.runtime.getURL('/viewfil.html?' + $.param(urlParams));

        chromeAPI.isExtensionUrlOpen(path, function (tabViewFil) {
            if (tabViewFil) {
                var update = {
                    active: true,
                    url: urlViewFil
                };
                chrome.tabs.update(tabViewFil.id, update);
            } else {
                chrome.tabs.create({
                    url: urlViewFil,
                    active: true
                });
            }
        });
    },
    /**
     * Add more refs (prev or next) to the swiper 
     */
    loadMoreRefs: function (swiper) {
        //when slides is removed in discover for exple
        if (swiper.slides.length === 0) {
            return;
        }
        var realIndex = swiper.realIndex,
            previousIndex = swiper.previousIndex,
            direction = realIndex < previousIndex ? 'prev' : 'next',
            linkId, idxPlaceholder, swiAddFunc,
            totalSlides = swiper.slides.length;

        if (direction === 'prev') {
            if ((totalSlides > 5) && realIndex > 1) {
                return;
            }
            idxPlaceholder = 0;
            swiAddFunc = 'prependSlide';
        } else {
            if ((totalSlides > 5) && realIndex < (totalSlides - 2)) {
                return;
            }
            idxPlaceholder = totalSlides - 1;
            swiAddFunc = 'appendSlide';
        }

        if ($(swiper.slides[idxPlaceholder]).hasClass('placeholder-slide')) {
            return;
        }
        linkId = $(swiper.slides[idxPlaceholder]).data('link-id');
        var filParams = swiper.params.filParams;
        if (linkId === filParams.firstRefLinkId) {
            return;
        }
        //avoid add remove place holder if few slides
        var $placeHolder;
        if (totalSlides > 5) {
            $placeHolder = this.contentPlaceholder();
            swiper[swiAddFunc]($placeHolder);
        }
        var user = swiper.params.user,
            urlParams = utils.urlParams(window.location.href);

        var params = {
            method: direction,
            user_cle: user.cle,
            link_id: linkId,
            filter: urlParams.filter,
            term: urlParams.term,
            selected_tags: urlParams.selected_tags
        };
        utils.debug(params);

        var self = this;
        bg.api.getFilData(params, function (res) {
            utils.debug(res);
            if (!res.error && res.refs.length > 0) {
                var refs = res.refs;
                var $ul = $('<ul></ul>');
                for (var i = 0, $slide; i < refs.length; i++) {
                    $slide = self.createSlide(refs[i]);
                    $ul.append($slide);
                }
                var slides = $ul.children();
                //will remove placeholder
                self.addSlides(slides.toArray(), swiper, direction);
                return;

            }
            if ($placeHolder) {
                idxPlaceholder = direction === 'prev' ? idxPlaceholder : idxPlaceholder + 1;
                swiper.removeSlide(idxPlaceholder);
            }
        });
    },
    /**
     * @param {Object} params parameters
     * @property {String} user_cle required
     * @property {String} filter for getting cross
     */
    initFilDates: function (params, idInitialRef, swiper) {
        bg.api.getFilDates(params, function (res) {
            utils.debug(res);
            if (res.error) {
                return;
            }
            if (res.dates.length > 10) {
                sliderBar.init(res.dates, idInitialRef, swiper);
            } else {
                sliderBar.destroy();
            }
        });
    },
    initFilterCross: function (swiper, $checkbox) {
        var urlParams = utils.urlParams(window.location.href),
            canChange = true,
            self = this,
            $tooltipCrossInner = $('.tooltip-cross .tooltip-inner');
        //init checkbox
        if ((urlParams.filter === urlParams.term) && urlParams.filter === 'cross') {
            $checkbox.prop('checked', true);
            $tooltipCrossInner.text(NDLConst.TOOLTIP_CHECKBOX_DISABLE_CROSS);
        } else {
            $checkbox.prop('checked', false);
            $tooltipCrossInner.text(NDLConst.TOOLTIP_CHECKBOX_SHOW_CROSS);
        }
        $checkbox.off('change').change(function () {
            var isChecked = $checkbox.prop('checked');
            if (!canChange) {
                this.checked = !isChecked;
                return;
            }
            urlParams = utils.urlParams(window.location.href);
            var state = {
                cle: swiper.params.cleUser
            };
            var $fil = $('.fil');
            if (isChecked) {
                urlParams.filter = 'cross';
                urlParams.term = 'cross';
                urlParams.selected_tags = null;
            } else {
                urlParams.filter = urlParams.term = null;
            }
            //copy
            state = Object.assign(state, urlParams);

            var filParams = {
                user_cle: urlParams.cle,
                link_id: urlParams.ref ? urlParams.ref : urlParams.link_id,
                filter: urlParams.filter,
                term: urlParams.term
            };
            utils.cleanUrlParams(state);
            canChange = false;
            self.filter(filParams, swiper, function (success) {
                if (success) {
                    self.search.resetSearchFilter();
                    $checkbox.prop('checked', isChecked);
                    var tooltipText;
                    if (isChecked) {
                        tooltipText = NDLConst.TOOLTIP_CHECKBOX_DISABLE_CROSS;
                    } else {
                        tooltipText = NDLConst.TOOLTIP_CHECKBOX_SHOW_CROSS;
                    }
                    $fil.data('filter', urlParams.filter);
                    $tooltipCrossInner.text(tooltipText);
                    history.replaceState(state, null, '?' + $.param(state));
                } else {
                    $checkbox.prop('checked', !isChecked);
                }
                setTimeout(function () {
                    canChange = true;
                }, 1000);
            });
        });
    },
    /**
     * search manager
     */
    search: {
        selectedTags: [],
        _$autocomplete: null,
        _$checkbox: null,
        _swiper: null,
        // _$searchBar: null,
        /**
         * check if a tag is selected, if not add it to list of
         * selectedTags
         * @param {string} tag
         */
        isTagSelected: function (tag) {
            tag = tag.toLowerCase();
            if (this.selectedTags.indexOf(tag) > -1) {
                return true;
            }
            this.selectedTags.push(tag);
            return false;
        },
        /**
         * Remove tag from selected tags
         * @param {string} tag
         */
        removeTag: function (tag) {
            var idxTag = this.selectedTags.indexOf(tag);
            if (idxTag == -1) {
                return;
            }
            this.selectedTags.splice(idxTag, 1);
        },
        init: function (swiper, $searchBar) {
            var $autocomplete = $searchBar.find('input'),
                $subHeader = $searchBar.closest('.sub-header'),
                $checkbox = $subHeader.find('.checkbox-cross'),
                self = this,
                urlParams;

            this._$autocomplete = $autocomplete;
            this._$checkbox = $checkbox;
            // this._$searchBar = $searchBar;
            this._swiper = swiper;

            $autocomplete.autocomplete({
                minLength: 2,
                appendTo: ".search-results .results",
                // autoFocus : true,
                create: function (event, ui) {
                    $('.ui-helper-hidden-accessible').remove();
                    //restore last selected tags
                    urlParams = utils.urlParams(window.location.href);
                    var prevSelectedTags = urlParams.selected_tags;
                    if (prevSelectedTags && prevSelectedTags.length > 0) {
                        self.selectedTags = prevSelectedTags;
                    }
                    self.appendTagsFilBody();
                },
                source: function (req, callback) {
                    var term = req.term ? req.term.trim() : '';
                    urlParams = utils.urlParams(window.location.href);
                    var searchParams = {
                        method: 'filter',
                        filter: 'search_tags',
                        term: term,
                        link_id: urlParams.link_id,
                        user_cle: urlParams.cle,
                        selected_tags: self.selectedTags
                    };
                    bg.api.getFilData(searchParams, function (res) {
                        utils.debug(res);
                        if (res.error) {
                            if (res.error === 'EMPTY_RESULT') {
                                var data = {
                                    noResult: true,
                                    tag: NDLConst.EMPTY_RESULT_SEARCH_TAGS
                                };
                                callback([data]);
                            }
                            return;
                        }
                        callback(res.results);
                    });
                },
                focus: function (event, ui) {
                    //disable events autocompletefocus
                    event.preventDefault();
                    var item = ui.item;
                    if (item.noResult) {
                        return;
                    }
                    $(this).val(item.tag);

                },
                select: function (e, ui) {
                    var item = ui.item,
                        tag = item.tag;
                    //no tag result found
                    if (item.noResult) {
                        return;
                    }
                    //add this tag to selected
                    self.isTagSelected(tag);
                    self.searchOK();
                }
            });

            $autocomplete.data('ui-autocomplete')._renderItem = function (ul, data) {
                var $li = $('<li class="result-item">')
                    .data('value', data)
                    .append(self.templateSearchTagResult(data))
                    .appendTo(ul);
                return $li;
            };
            $autocomplete.keydown(function (e) {
                if (e.keyCode === 13) {
                    var $searchResults = $('.search-results .tag-result.ui-state-active:not(.no-result)');
                    if ($searchResults.length > 0) {
                        return;
                    }
                    self.selectedTags = [];
                    self.searchOK();
                }
            });
            $searchBar.find('.search-btn').click(function () {
                self.selectedTags = [];
                self.searchOK();
            });

            $('.reset-filter').click(function () {
                urlParams = utils.urlParams(window.location.href);
                if (urlParams.filter !== 'search') {
                    //already desactivated
                    return;
                }
                //params with no filter to desactivate it
                var params = {
                    method: 'filter',
                    link_id: urlParams.ref || urlParams.link_id,
                    user_cle: urlParams.cle
                };
                commons.filter(params, swiper, function (success) {
                    if (success) {
                        self.resetSearchFilter();
                    }
                });
            });
        },
        appendTagsFilBody: function () {
            var self = this,
                tags = self.selectedTags;

            var $selectedTagsWraper = $('.fil-body .selected-tags-wrapper'),
                $selectedTagsInBody = $selectedTagsWraper.find('.selected-tags'),
                $ulTags = $selectedTagsWraper.find('ul'),
                $btnResetFilter = $selectedTagsWraper.find('.reset-filter');
            $ulTags.empty();
            if (tags.length > 0) {
                //append tags
                for (var i = 0, tag; i < tags.length; i++) {
                    tag = tags[i];
                    var $tagItem = self.templateTags(tag);
                    $ulTags.append($tagItem);
                }
                $btnResetFilter.addClass('ndl-active');

            } else {
                // append term
                var urlParams = utils.urlParams(window.location.href);
                if ((urlParams.filter === 'search') && urlParams.term) {
                    $tagItem = self.templateTags(urlParams.term);
                    $tagItem.addClass('term');
                    $ulTags.append($tagItem);
                }
                $btnResetFilter.removeClass('ndl-active');
            }
            var $fil = $('.fil');
            $ulTags.find('.remove-tag').click(function () {
                $fil.removeClass('search-bar-open');
                var $this = $(this),
                    $li = $this.closest('li'),
                    tag = $li.find('.tag-name').text();


                var urlParams = utils.urlParams(window.location.href),
                    cpySelectedTags = self.selectedTags.slice(),
                    idxTag = cpySelectedTags.indexOf(tag);
                //copy it, in case of error tags is not removed from selected_tags
                cpySelectedTags.splice(idxTag, 1);
                var searchParams = {
                    method: 'filter',
                    filter: 'search',
                    term: urlParams.term,
                    link_id: urlParams.link_id || urlParams.ref, //ref for curr user
                    user_cle: urlParams.cle,
                    selected_tags: cpySelectedTags
                };

                if (cpySelectedTags.length === 0) {
                    searchParams.filter = searchParams.term = null;
                    utils.cleanUrlParams(searchParams);
                }
                //update swiper when tag is removed from fil body tags
                commons.filter(searchParams, self._swiper, function (success) {
                    if (success) {
                        self.removeTag(tag);
                        if (self.selectedTags.length === 0) {
                            $btnResetFilter.removeClass('ndl-active');
                        }
                        urlParams.selected_tags = self.selectedTags;
                        //it was a search by term, no tag was selected
                        if ($li.hasClass('term') || self.selectedTags.length === 0) {
                            urlParams.term = urlParams.filter = null;
                        }
                        $li.remove();
                        utils.cleanUrlParams(urlParams);
                        history.replaceState(urlParams, null, '?' + $.param(urlParams));
                    }
                });

            });

            setTimeout(function () {
                $selectedTagsInBody.scrollTop($selectedTagsInBody[0].scrollHeight);
            }, 100);
        },
        searchOK: function () {
            var self = this,
                $autocomplete = self._$autocomplete,
                $checkbox = self._$checkbox,
                swiper = self._swiper,
                term = $autocomplete.val().trim();

            if (self.selectedTags.length === 0 && term.length < 2) {
                utils.debug('Nothing to search');
                return;
            }

            var urlParams = utils.urlParams(window.location.href);
            var searchParams = {
                method: 'filter',
                filter: 'search',
                term: term,
                link_id: urlParams.link_id || urlParams.ref, //ref for curr user
                user_cle: urlParams.cle,
                selected_tags: self.selectedTags
            };
            urlParams.filter = 'search';
            urlParams.term = term;
            urlParams.selected_tags = self.selectedTags;

            $checkbox.prop('checked', false);
            commons.filter(searchParams, swiper, function (success) {
                if (success) {
                    $('.fil').removeData('filter');
                    utils.cleanUrlParams(urlParams);
                    history.replaceState(urlParams, null, '?' + $.param(urlParams));
                    self.appendTagsFilBody();
                }
                // else {
                //     no data, remove this tag
                //     self.removeTag(term);
                // }
            });

        },
        resetSearchFilter: function () {
            var $selectedTagsWraper = $('.fil-body .selected-tags-wrapper');
            this.selectedTags = [];
            $selectedTagsWraper.find('ul').empty();
            $selectedTagsWraper.find('.reset-filter')
                .removeClass('ndl-active');
            this._$autocomplete.val('');
            var $checkbox = this._$checkbox;
            if ($checkbox) {
                $checkbox.prop('checked', false);
            }
            var urlParams = utils.urlParams(window.location.href);
            urlParams.filter = urlParams.term = urlParams.selected_tags = null;
            utils.cleanUrlParams(urlParams);

            history.replaceState(urlParams, null, '?' + $.param(urlParams));
        },
        templateTags: function (tagname) {
            var t = `<li>
                    <a class="remove-tag"><i class="la la-remove"></i></a>
                    <span class="tag-name">${tagname}</span>
                </li>`;
            return $(t);
        },
        templateSearchTagResult: function (data) {
            var tagname = data.tag;
            var t = `<div class="tag-result ${data.noResult? 'no-result': ''}">
                <div class="tag-name">${tagname}</div>
            </div>`;

            return t;
        },
        triggerHashtagsFilter: function (hashtag) {
            var $autocomplete = this._$autocomplete;
            if (null === $autocomplete) {
                return;
            }
            $autocomplete.val(hashtag);
            this.isTagSelected(hashtag);
            this.searchOK();
        }

    },
    filter: function (filterParams, swiper, cb) {
        utils.debug('filter params==> ', filterParams);

        var $filBody = $('.fil-body'),
            self = this;
        filterParams.method = 'filter';
        $filBody.addClass('loading');

        bg.api.getFilData(filterParams, function (res) {
            utils.debug(res);
            // return
            $filBody.removeClass('loading');
            if (res.error) {
                cb && cb();
                var type = 'error';
                if (res.error === 'EMPTY_RESULT') {
                    $('.ndl-message-success').text(res.message);
                    type = 'success';
                } else {
                    $('.ndl-message-error').text(res.message || NDLConst.ERR_GLOBAL);
                }
                utils.showHideMessage(type);
                return;
            }
            cb && cb(true);
            var refs = res.refs;
            var initialSlide = -1;
            var $ul = $('<ul></ul>');
            for (var i = 0, $slide; i < refs.length; i++) {
                $slide = self.createSlide(refs[i]);
                $ul.append($slide);
                if (refs[i].link_id.toString() === filterParams.link_id.toString()) {
                    initialSlide = i;
                }
            }
            // if (refs.length > 0 && initialSlide === 0) {
            if (refs.length > 0 && initialSlide === -1) {
                initialSlide = Math.floor(refs.length / 2);
            }

            var slides = $ul.children();
            //will remove placeholder
            self.addSlides(slides.toArray(), swiper);
            swiper.slideTo(initialSlide);
            swiper.update();
            var id_ref = refs[initialSlide].id;
            self.initFilDates(filterParams, id_ref, swiper);
        });

    },
    updateRefLegende: function (ref, $slideContent) {
        var $fil = $('.fil');
        if ($fil.length === 0) {
            return;
        }
        var $popupUpdateLegende = this.template.popupEditLegende();
        var $popupBody = $popupUpdateLegende.find('.popup-body'),
            $textarea = $popupBody.find('textarea'),
            $legende = $slideContent.find('.legende'),
            oldLegende = $legende.children(':not(.placeholder-legende)').text(),
            $errorMsg = $popupBody.find('.ndl-message-error'),
            self = this;
        $fil.append($popupUpdateLegende);
        $popupUpdateLegende.addClass('active');
        $textarea.val(oldLegende).focus();

        $popupBody.draggable({
            addClasses: false,
            containment: '.fil',
            handle: '.header'
        });

        $textarea.keyup(function () {
            $popupUpdateLegende.removeClass('error');
            var newLegende = $textarea.val().trim();
            if (newLegende.length > 240) {
                $errorMsg.text('240 caractères maximum autorisés.');
                $popupUpdateLegende.addClass('error');
            }
        });

        var $btnValidateEdit = $popupUpdateLegende.find('.btn-validate-edit-title');
        $btnValidateEdit.click(function () {

            var newLegende = $textarea.val().trim();
            if (!newLegende || newLegende.trim().length === 0) {
                $errorMsg.text('Ce Champ est requis.');
                $popupUpdateLegende.addClass('error');
                return;
            }

            if (newLegende.length > 240) {
                utils.debug('max length legende autorised 240 chars');
                return;
            }
            if (newLegende === oldLegende) {
                return;
            }

            var $this = $(this);
            newLegende = utils.htmlEscape(newLegende);

            var data = {
                id: ref.id,
                link_id: ref.link_id,
                legende: newLegende
            };
            $this.addClass('loading');
            bg.api.updateLegende(data, function (res) {
                utils.debug(res);
                $this.removeClass('loading');
                if (!res.error) {
                    if (res.updated) {
                        $popupUpdateLegende.remove();

                        ref.legende = newLegende;
                        // newLegende = utils.autoLinkHashtags(newLegende);
                        console.log(newLegende);

                        $legende.html('<div class="content">' + utils.autoLinkHashtags(newLegende) + '</div>');
                        // $legende.html('<div class="content"></div>');
                        // $legende.append(newLegende);
                        $legende.find('.ndl-hashtag').click(function (e) {
                            e.stopPropagation();
                            var hastag = $(this).text();
                            self.search.triggerHashtagsFilter(hastag);
                        });
                        $legende.effect('highlight', {
                            duration: 1000
                        });
                        bg.tabsManager.updateTabData(data);
                        //to update monfil if it's open in a tab
                        var message = {
                            subject: 'update_legende',
                            body: data
                        };
                        bg.messaging.broadcast(message);
                    }
                } else {
                    $errorMsg.text(res.message || NDLConst.ERR_GLOBAL);
                    $popupUpdateLegende.removeClass('success').addClass('error');
                }
            });
        });

        $popupUpdateLegende.find('.close-popup-btn, .header .close')
            .click(function () {
                $popupUpdateLegende.remove();
            });

        $textarea.focusin(function () {
            $popupUpdateLegende.removeClass('error success');
        });

        $(document).off('keyup').keyup(function (e) {
            if (e.keyCode === 27) {
                $popupUpdateLegende.remove();
            }
        });

        $popupUpdateLegende.click(function (e) {
            if ($('.popup-content-wrapper', this).is($(e.target))) {
                $popupBody.effect('shake', {
                    times: 2,
                    distance: 5
                });
            }
        });

    },
    /**
     * Follow / unfollow a fil by memorizing it from a selected ref
     */
    memoriseRef: function (isMemorized, dataRef) {
        var $filBody = $('.fil-body');
        if ($filBody.hasClass('memorizing')) {
            return;
        }
        var data = {
            id_ref: dataRef.id,
            link_id: dataRef.link_id,
            user_cle: dataRef.user_cle,
            method: isMemorized ? 'unfollow' : 'follow'
        };

        $filBody.addClass('memorising');
        var self = this;
        bg.api.userActionPost(data, function (res) {
            utils.debug(res);
            $filBody.removeClass('memorizing');
            if (res.error) {
                $('.ndl-message-error').text(res.message || NDLConst.ERR_GLOBAL);
                utils.showHideMessage('error');
                return;
            }
            self.setMemorizeRefSate(res.is_following, dataRef);
            if (res.is_following) {
                sliderBar.setSliderMarker(dataRef, sliderBar.markers.MEMORIZED);
            } else {
                sliderBar.hideMarker(sliderBar.markers.MEMORIZED);
            }
            commons.setFilColor(res.fil_colors, dataRef.user_cle);
            //notify opened fil in content script data is changed
            var changedData = {
                id: dataRef.id,
                linkId: dataRef.link_id,
                userCle: dataRef.user_cle,
                isMemorized: res.is_following,
                filColors: res.fil_colors
            };
            bg.tabsManager.notifyCsFilsDataChanged('memorized-ref', changedData);


        });
    },
    /**
     * Change state of memorized ref
     */
    setMemorizeRefSate: function (isMemorized, ref) {
        var $fil = $('.fil'),
            idRef = ref ? ref.id : null,
            $slideContent = $fil.find(
                '.swiper-slide[data-id="' + idRef + '"] .slide-content');

        if (isMemorized) {
            $fil.addClass('following');
            $fil.find('.slide-content').removeClass('memorized');
            $slideContent.addClass('memorized')
                .find('.memorise-fil-ref')
                .attr('data-title', NDLConst.TITLE_REF_MEMORIZED);
        } else {
            $fil.removeClass('following');
            $fil.find('.slide-content').removeClass('memorized');
            $slideContent.find('.memorise-fil-ref')
                .attr('data-title', NDLConst.TITLE_REF_NOT_MEMORIZED);
            $fil.removeData('memorized-ref');
        }
        if (idRef && isMemorized) {
            $fil.data('memorized-ref', ref);
        }

    },
    deleteRef: function (ref) {
        var $fil = $('.fil');
        if ($fil.length === 0) {
            return;
        }
        var $popupDelRef = this.template.popupDeleteRef();

        var $popupBody = $popupDelRef.find('.popup-body'),
            $msgContent = $popupBody.find('.message-content'),
            $message = $('<div></div>'),
            $retryPannel = $(`<div class="retry-message">Vous n'avez pas reçu de mail ?
                <a class= "retry-sendmail-btn">cliquer ici</a> pour renvoyer </div>`);

        $fil.append($popupDelRef);
        $popupDelRef.addClass('active');

        $message.removeClass().addClass('info').text(NDLConst.DEL_REF_INFO);
        $msgContent.html($message);

        var $btnSendMail = $popupDelRef.find('.btn-send-mail');
        var isSending = false;
        var data = {
            retry: false,
            ref: ref
        };
        $popupBody.draggable({
            addClasses: false,
            containment: '.fil',
            handle: '.header'
        });
        $btnSendMail.click(function () {
            if (isSending) {
                return;
            }
            isSending = true;
            $popupDelRef.addClass('sending');
            bg.api.deleteRef(data, function (res) {
                data.retry = false;
                utils.debug(res);
                $popupDelRef.removeClass('info sending');
                isSending = false;
                if (!res.error) {
                    return $message.removeClass().addClass('sent')
                        .text(res.message || NDLConst.DEL_REF_CONFIRMATION);
                }
                if (res.error === 'REF_ALREADY_MARKED_FOR_DELETE') {
                    $message.removeClass().addClass('sent').text(res.message);
                    $message.append($retryPannel);
                    $retryPannel.find('.retry-sendmail-btn')
                        .off('click').click(function () {
                            data.retry = true;
                            $btnSendMail.trigger('click');
                        });
                    return;
                }
                $message.removeClass().addClass('error').text(res.message || NDLConst.ERR_GLOBAL);
                $msgContent.html($message);
            });
        });

        $popupDelRef.find('.close-popup-btn, .header .close')
            .click(function () {
                if ($popupDelRef.hasClass('sending')) {
                    return;
                }
                $popupDelRef.remove();
            });
        $(document).off('keyup').keyup(function (e) {
            if (e.keyCode === 27) {
                $popupDelRef.remove();
            }
        });
        $popupDelRef.click(function (e) {
            if ($('.popup-content-wrapper', this).is($(e.target))) {
                $popupBody.effect('shake', {
                    times: 2,
                    distance: 5
                });
            }
        });
    }
};

/**
 * slider bar manager
 */

var sliderBar = {
    markers: {
        DISCOVERED: 1,
        MEMORIZED: 2,
        dataDiscover: {},
        dataMemorized: {},
    },
    $tooltipRefCount: $('<div class="tooltip-slider tooltip-slider-refs"></div>'),
    $tooltipDate: $('<div class="tooltip-slider tooltip-slider-date"></div>'),
    $startDate: null,
    $endDate: null,
    _swiper: null,
    _slider: null,
    _dates: null,
    templateSlider: function () {
        var $template = $(`<div class="slider-info-inner">
            <div class="slider-date slider-start-date"></div>
            <div class="slider-wraper">
                <div class="slider-container">
                    <div id="slider"></div>
                    <div class="marker-slider marker-discovered">
                        <button class="marker-btn">Nouveau croisement</button>
                    </div>
                    <div class="marker-slider marker-memorized">
                        <button class="marker-btn">Mémorisé</button>
                    </div>   
                </div>
            </div>   
            <div class="slider-date slider-end-date"></div>
            </div>`);

        return $template;
    },
    init: function (dates, idInitialRef, theSwiper) {
        var $sliderInfo = $('.slider-info'),
            $slider = $sliderInfo.find('#slider'),
            slider = $slider[0],
            initial = utils.findIndexRef(dates, 'id', idInitialRef),
            self = this;

        if (!slider || !slider.noUiSlider) {
            $sliderInfo.append(this.templateSlider());
            slider = $sliderInfo.find('#slider')[0];
            var sliderMax = dates.length - 1;

            noUiSlider.create(slider, {
                start: initial,
                animate: false,
                behaviour: 'tap-hover',
                range: {
                    min: 0,
                    max: sliderMax
                }
            });
            var $handle = $sliderInfo.find('.noUi-handle');
            $handle.append([this.$tooltipRefCount, this.$tooltipDate]);

            this.$startDate = $sliderInfo.find('.slider-start-date');
            this.$endDate = $sliderInfo.find('.slider-end-date');

            this._swiper = theSwiper;
            this._slider = slider;
            var timeout;
            //add transition event on swiper to sync with slider
            theSwiper.on('transitionStart', function () {
                clearTimeout(timeout);
            });
            theSwiper.on('transitionEnd', function () {
                //update handle after slider is completely stoped
                timeout = setTimeout(function () {
                    self.updateHandlePosition();
                }, 10);
            });

        } else {
            var newOptions = {
                range: {
                    min: 0,
                    max: dates.length - 1
                },
                start: initial
            };
            slider.noUiSlider.off();
            slider.noUiSlider.updateOptions(newOptions);
        }

        var startedIdx = -1,
            stopedIdx = initial;
        this._dates = dates;
        this.updateSliderUI(initial);

        $sliderInfo.find('.slider-start-date, .slider-end-date')
            .off('click')
            .click(function () {
                var $this = $(this),
                    currentIdx = 0,
                    currentHandlePos = slider.noUiSlider.get();

                currentHandlePos = Math.floor(currentHandlePos);
                if ($this.hasClass('slider-end-date')) {
                    currentIdx = dates.length - 1;
                }
                if (currentIdx === currentHandlePos) {
                    return;
                }
                self.setHandlePos(currentIdx);
                self.sliderPositionChanged(currentIdx);


            });

        slider.noUiSlider.on('update', function (values, handle) {
            var currentIdx = Math.floor(values[handle]);
            self.updateSliderUI(currentIdx);

        });

        slider.noUiSlider.on('start', function (values, handle) {
            startedIdx = Math.floor(values[handle]);
        });

        slider.noUiSlider.on('change', function (values, handle, unencoded, tap) {
            var currentIdx = Math.floor(values[handle]);
            stopedIdx = currentIdx;
            if (startedIdx >= 0 && stopedIdx >= 0 && (startedIdx === stopedIdx) && !tap) {
                //do not get data for same idx
                return;
            }
            self.sliderPositionChanged(currentIdx);
            startedIdx = currentIdx;
        });

        //add discover marker
        var discoverLinkId = $('body.discover .cross-link-info a').data('link-id');
        var markers = self.markers,
            userCle = theSwiper.params.cleUser,
            data;
        if (discoverLinkId) {
            var dataDiscoverMarker = markers.dataDiscover;
            if (dataDiscoverMarker.link_id === discoverLinkId && dataDiscoverMarker.user_cle === userCle) {
                self.setSliderMarker(dataDiscoverMarker, markers.DISCOVERED);
            } else {
                //Need to get ref id for this user using discover link_id to 
                //discover marker
                data = {
                    method: 'ref_info',
                    user_cle: userCle,
                    link_id: discoverLinkId
                };
                bg.api.getFilData(data, function (res) {
                    if (res.error) {
                        return;
                    }
                    self.markers.dataDiscover = res.ref;
                    self.setSliderMarker(res.ref, self.markers.DISCOVERED);
                });

            }
        }
        //add memorized marker
        var $fil = $('.fil'),
            ref = $fil.data('memorized-ref');
        if ($fil.hasClass('following') && ref && ref.id) {
            sliderBar.setSliderMarker(ref, markers.MEMORIZED);
        }
    },
    /**
     * update slider dates and refs count values
     */
    updateSliderUI: function (handlePos) {
        var dates = this._dates,
            len = dates.length;
        var startDate = utils.sliderDate(dates[0].created_at),
            endDate = utils.sliderDate(dates[len - 1].created_at),
            handleDate = utils.sliderDate(dates[handlePos].created_at),
            handleRefCount = (handlePos + 1) + ' / ' + len;

        this.$startDate.text(startDate);
        this.$endDate.text(endDate);
        this.$tooltipRefCount.text(handleRefCount);
        this.$tooltipDate.text(handleDate);

    },
    /**
     * Sync swiper and and handle when swiper move
     */
    updateHandlePosition: function () {
        var slider = this._slider;

        if (!slider || !slider.noUiSlider) {
            //for exple it is destroyed when filtering
            return;
        }
        var swiper = this._swiper;
        var realIndex = swiper.realIndex,
            activeSlide = $(swiper.slides[realIndex]),
            idRef = activeSlide.data('id'),
            dates = this._dates,
            currentHandlePos = slider.noUiSlider.get();

        currentHandlePos = Math.floor(currentHandlePos);
        if (!idRef || !dates[currentHandlePos]) {
            return;
        }
        if (idRef === dates[currentHandlePos].id) {
            return;
        }
        var handlePos = utils.findIndexRef(dates, 'id', idRef);
        this.setHandlePos(handlePos);
    },
    /**
     * move handle to position with animation when not tap
     */
    setHandlePos: function (handlePos) {
        if (handlePos < 0) {
            return;
        }
        var $slider = $('#slider');
        $slider.addClass('noUi-state-tap');
        this._slider.noUiSlider.set(handlePos);
        setTimeout(function () {
            $slider.removeClass('noUi-state-tap');
        }, 700);
    },
    /**
     * Handle released! fetch data corresponding to this position
     */
    sliderPositionChanged: function (currentIdx) {
        var idRef = this._dates[currentIdx].id,
            dateRef = this._dates[currentIdx].created_at,
            swiper = this._swiper,
            $swiperContainer = $(swiper.el),
            $newActiveSlide = $swiperContainer.find('[data-id=' + idRef + ']');
        var idxRef = $newActiveSlide.index(),
            urlParams = utils.urlParams(window.location.href);

        if (idxRef > -1) {
            var state = urlParams;
            state.ref = $newActiveSlide.data('link-id');
            history.replaceState(state, null, '?' + $.param(state));
            return swiper.slideTo(idxRef);
        }

        var data = {
            method: 'slider',
            user_cle: this._swiper.params.cleUser,
            id_ref: idRef,
            date_ref: dateRef,
            filter: urlParams.filter,
            term: urlParams.term,
            selected_tags: urlParams.selected_tags
        };
        var $filBody = $('.fil-body');
        $filBody.addClass('loading');

        bg.api.getFilData(data, function (res) {
            utils.debug(res);
            if (!res.error && res.refs.length > 0) {
                var refs = res.refs;
                var $ul = $('<ul></ul>');
                for (var i = 0, $slide; i < refs.length; i++) {
                    $slide = commons.createSlide(refs[i]);
                    $ul.append($slide);
                }
                var slides = $ul.children(),
                    initialSlide = utils.findIndexRef(refs, 'id', idRef);
                commons.addSlides(slides.toArray(), swiper);

                swiper.slideTo(initialSlide);
                var activeRef = refs[initialSlide],
                    state = utils.urlParams(window.location.href);

                //link_id of the active ref
                state.ref = activeRef.link_id;
                history.replaceState(state, null, '?' + $.param(state));
            } else {
                $('.ndl-message-error').text(res.message || NDLConst.ERR_GLOBAL);
                utils.showHideMessage('error');
            }
            $filBody.removeClass('loading');
        });
    },
    /**
     * Remove a ref from slider (e.g.: when a ref is deleted from fil)
     */
    removeSliderRef: function (idRef) {
        var dates = this._dates,
            swiper = this._swiper;
        if (!dates || !swiper) {
            return;
        }
        var idxRef = dates.findIndex(function (ref) {
            return ref.id === idRef;
        });
        if (idxRef < 0) {
            return;
        }
        dates.splice(idxRef, 1);
        this.updateSliderUI(swiper.realIndex);
    },
    /**
     * destroy slider
     */
    destroy: function () {
        var swiper = this._swiper,
            slider = this._slider;
        if (swiper) {
            swiper.off('transitionStart');
            swiper.off('transitionEnd');
        }
        if (slider && slider.noUiSlider) {
            slider.noUiSlider.destroy();
            $(slider).closest('.slider-info').empty();
        }
    },
    /**
     * Add marker point on slider bar,
     * Allow mark current discovered and memorised refs
     * on slider bar to go back easily
     */
    setSliderMarker: function (dataMarker, markerType) {
        var dates = this._dates,
            swiper = this._swiper;
        if (null === dates || null === swiper) {
            return;
        }
        var idxRef = dates.findIndex(function (ref) {
            return ref.id === dataMarker.id;
        });
        var $marker,
            $markerD = $('.marker-discovered'),
            $markerM = $('.marker-memorized'),
            markers = this.markers;
        if (markers.DISCOVERED === markerType) {
            $marker = $markerD;
        } else {
            $marker = $markerM;
        }
        if (idxRef < 0) {
            $marker.removeClass('active');
            return;
        }

        var min = 0,
            max = dates.length - 1,
            range = [min, max];
        var percentValue = utils.fromIndexToPercentage(range, idxRef);

        $marker.css({
            left: 'calc(' + percentValue + '% - ' + $marker.width() / 2 + 'px )'
        }).addClass('active');


        this.isElWidthOverlap($markerD, $markerM);

        var $markerBtn = $marker.find('.marker-btn'),
            $swiperContainer = $(swiper.el),
            idRef = dates[idxRef].id,
            isHighlighting = false,
            self = this;
        $markerBtn.off('click').click(function (e) {
            e.stopPropagation();
            var $slide = $swiperContainer.find('[data-id=' + idRef + ']');
            if ($slide.length > 0) {
                var urlParams = utils.urlParams(window.location.href);
                history.replaceState(urlParams, null, '?' + $.param(urlParams));
                var $slideContent = $slide.find('.slide-content');
                //remove clamp for firefox before highlighting
                var $elWithClamps = $slideContent.find('.clamp');
                $elWithClamps.removeClass('clamp');
                var indexSlide = $slide.index();
                swiper.slideTo(indexSlide);
                if (!isHighlighting) {
                    isHighlighting = true;
                    $slideContent.effect('highlight', {
                        duration: 1000,
                        color: '#00B2B2'
                    }, function () {
                        $elWithClamps.addClass('clamp');
                        isHighlighting = false;
                    });
                }
            } else {
                self.sliderPositionChanged(idxRef);
            }
        });
    },
    /**
     * Hide a marker
     */
    hideMarker: function (markerType) {
        var $marker,
            markersType = this.markers;
        if (markersType.DISCOVERED === markerType) {
            $marker = $('.marker-discovered');
        } else if (markersType.MEMORIZED === markerType) {
            $marker = $('.marker-memorized');
        } else {
            return $('.marker-slider').removeClass('active');
        }
        $marker.off('click').removeClass('active');
    },
    /**
     * check if two elements (slider markers) overlaps, if so
     * augment margin top of second element
     */
    isElWidthOverlap: function ($el1, $el2) {
        if (!$el1.is(':visible') || !$el2.is(':visible')) {
            return;
        }
        var leftPosEl1 = ($el1.position().left),
            leftPosEl2 = ($el2.position().left),
            widthEl1 = $el1.width(),
            widthEl2 = $el2.width();
        if (leftPosEl1 < (leftPosEl2 + widthEl2) &&
            (leftPosEl1 + widthEl1) > leftPosEl2) {
            $el2.addClass('overlaps');
        } else {
            $el2.removeClass('overlaps');
        }

    }

};

/**
 * messaging here to allow update when fil data change
 */
var messaging = {
    init: function () {
        this.onMessage();
    },
    onMessage: function () {
        chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {
            var subject = request.subject;
            if (subject && processIncomingMessagesCommons[subject]) {
                processIncomingMessagesCommons[subject](request, sender, sendResponse);
                return true;
            }
        });
    }
};
var processIncomingMessagesCommons = {
    /**
     * Update ref legende only on current user fil
     * (unique idRef is used)
     */
    update_legende: function (req, sender, sendResp) {
        var body = req.body;
        var idRef = body.id,
            $fil = $('.fil');
        utils.debug('live update legende', body);
        var $slide = $fil.find('[data-id=' + idRef + ']');
        if ($slide.length > 0) {
            var $legende = $slide.find('.legende');
            var legende = utils.autoLinkHashtags(body.legende);
            $legende.html('<div class="content">' + legende + '</div>');
            $legende.find('.ndl-hashtag').click(function (e)  {
                e.stopPropagation();
                var hastag = $(this).text();
                commons.search.triggerHashtagsFilter(hastag);
            });
        }
    },
    /**
     * Live update slider dates only on current user fil when ref is added
     */
    update_slider_dates: function (req, sender, sendResp) {
        var body = req.body,
            currentUserCle = body.user_cle,
            dates = body.dates,
            swiper = sliderBar._swiper;
        utils.debug('Live update slider dates ', dates);

        if (null === swiper) {
            return;
        }
        var userCle = swiper.params.user.cle;
        if (!dates || dates.length === 0 || userCle !== currentUserCle) {
            return;
        }
        var $fil = $('.fil');
        if ($fil.length === 0) {
            return;
        }
        var urlParams = utils.urlParams(window.location.href);
        if (urlParams.filter) {
            return;
        }
        utils.debug('live update slider dates', currentUserCle, dates);
        var activeIndx = swiper.realIndex;
        var idInitialRef = $(swiper.slides[activeIndx]).data('id');
        sliderBar.init(dates, idInitialRef, swiper);
    },
    on_ref_deleted: function (req, sender, sendResp) {
        var body = req.body,
            idRef = body.id_ref,
            swiper = commons.getSwiperInstance();

        if (!swiper || !idRef) {
            return;
        }
        var $swiperContainer = $(swiper.el),
            $slide = $swiperContainer.find('[data-id=' + idRef + ']'),
            idxRef = $slide.index();
        if (idxRef < 0) {
            return;
        }
        swiper.removeSlide(idxRef);
        swiper.update();
        sliderBar.removeSliderRef(idRef);

    },
    /**
     * Memorized ref is changed from content script
     */
    on_memorized_ref_changed_from_cs: function (req, sender, sendResp) {
        var body = req.body,
            userCle = body.user_cle,
            $fil = $('.fil-' + userCle);

        if ($fil.length === 0) {
            //it's not the good fil
            return;
        }
        utils.debug('Live update memorized ref', body);
        var markers = sliderBar.markers,
            isMemorized = body.isMemorized,
            filColors = body.fil_colors;

        delete body.isMemorized;
        delete body.fil_colors;

        if (isMemorized) {
            sliderBar.setSliderMarker(body, markers.MEMORIZED);
        } else {
            sliderBar.hideMarker(markers.MEMORIZED);
        }
        commons.setMemorizeRefSate(isMemorized, body);
        commons.setFilColor(filColors, body.user_cle);
    }
};

messaging.init();