var bg = window.chrome.extension.getBackgroundPage();

var discover = {
    /**Unique discover swiper */
    swiper: null,
    /**users key who referenced this link */
    cles: [],
    /**user details of displayed fil */
    activeUser: null,
    /**
     * Get discover url parameters
     * @returns {Object} params
     */
    tabDiscoverUrlParams: function () {
        var url = window.location.href,
            parsedUri = utils.parseUri(url),
            sort = parsedUri.queryKey.sort_by || 'created_at-asc',
            cle = parsedUri.queryKey.cle || null;

        var params = {
            cle: cle,
            linkId: parsedUri.queryKey.link_id,
            ref: parsedUri.queryKey.ref,
            sort: sort.split('-')[1].toLowerCase(),
        };
        return params;
    },
    /**
     * From discover url params, determine actual or next user fil
     * to show in the list of user cles
     */
    discoverUser: function () {
        var cles = this.cles;
        var urlParams = this.tabDiscoverUrlParams();
        var idxUser = urlParams.cle ? cles.indexOf(urlParams.cle) : -1;

        if (idxUser < 0) {
            idxUser = (urlParams.sort === 'asc') ? 0 : cles.length - 1;
        }
        return {
            linkId: urlParams.link_id,
            userCle: cles[idxUser],
            ref: urlParams.ref,
            sort: urlParams.sort,
            idxUser: idxUser
        };
    },
    init: function () {
        var urlParams = utils.urlParams(window.location.href);

        var linkId = urlParams.link_id;
        if (!linkId || !utils.isNumeric(linkId)) {
            return commons.showPageMessage(
                NDLConst.ERR_NO_DISCOVER_LINK_FOUND, 'info');
        }
        var data = {
            link_id: linkId
        };
        var self = this;
        bg.api.getLinkInfo(data, function (res) {
            utils.debug(res);
            if (res.error) {
                return commons.showPageMessage(
                    res.message || NDLConst.ERR_GLOBAL, 'error');
            }
            self.discover(res.data);
        });

    },
    /**
     * Update cross header with giving link and show it 
     */
    showCrossHeader: function (linkData) {
        var localRef = linkData.local_ref;
        if (linkData.referenced) {
            localRef--;
        }
        var href = linkData.url,
            title = linkData.title,
            linkId = linkData.link_id;
        var linkInfo =
            `<div class="cross-link-info">
                <h3>Vous croisez <span class="cross-fil-count"> ${localRef} </span>
                   fil${localRef> 1? 's': ''} sur la page 
                </h3>
                <a href=${href} target="_blank" rel="noopener" data-link-id=${linkId}>
                    ${title}
                </a>
            </div>`;
        $('.cross-link-container').append(linkInfo).addClass('ndl-active');
    },
    /**
     * get users ids whos referenced this link and init swiper
     * with first user, init fil actions too
     */
    discover: function (linkData) {
        var params = {
            link_id: linkData.link_id
        };
        var $pageContent = $('.page-content'),
            $filSortBtn = $('.fil-sort-btn');
        $pageContent.addClass('loading');
        var self = this;
        bg.api.discover(params, function (resUsers) {
            utils.debug(resUsers);
            if (resUsers.error || resUsers.cles.length === 0) {
                $pageContent.removeClass('loading');
                return commons.showPageMessage(resUsers.message || NDLConst.ERR_GLOBAL);
            }
            self.cles = resUsers.cles;
            var dataDiscover = self.discoverUser();
            var userCle = dataDiscover.userCle;

            $('.fil').addClass('fil-' + userCle);
            $filSortBtn.removeClass('fil-asc fil-desc')
                .addClass('fil-' + dataDiscover.sort);
            var swiper = commons.createSwiper(userCle);
            if (null === swiper) {
                utils.debug('ERR_INIT_SWIPER');
                $pageContent.removeClass('loading');
                return commons.showPageMessage(resUsers.message || NDLConst.ERR_GLOBAL);
            }
            var nbUsers = resUsers.cles.length;
            // if (dataDiscover.sort === 'asc' && (userCle === resUsers.cles[nbUsers - 1])) {
            //     $('.btn-hide').hide();
            // }
            // if (dataDiscover.sort === 'desc' && (userCle === resUsers.cles[0])) {
            //     $('.btn-hide').hide();
            // }

            self.swiper = swiper;
            var urlParams = utils.urlParams(window.location.href);
            var state = {
                link_id: urlParams.link_id,
                cle: userCle,
                sort_by: urlParams.sort_by,
                filter: urlParams.filter,
                term: urlParams.term,
                selected_tags: urlParams.selected_tags,
                ref: urlParams.ref //ref is link_id of active ref
            };
            state = utils.cleanUrlParams(state);

            history.replaceState(state, null, 'discover.html?' + $.param(state));

            //take active ref linkid in case of reload
            var linkId = urlParams.ref ? urlParams.ref : linkData.link_id;
            var filParams = {
                user_cle: userCle,
                link_id: linkId,
                filter: urlParams.filter,
                term: urlParams.term,
                selected_tags: urlParams.selected_tags
            };
            self.loadFil(filParams, function () {
                self.showCrossHeader(linkData);
                self.filActionManager();
                // var $searchBar = $('.search-bar');
                var $searchBar = $('.searchbar');
                commons.search.init(swiper, $searchBar);
                //init cross filter
                var $checkbox = $('.checkbox-cross');
                commons.initFilterCross(self.swiper, $checkbox);
            });
            // Init fil sort button
            if (resUsers.cles.length > 1) {
                $filSortBtn.click(function () {
                    self.filSorter(resUsers.cles[0], resUsers.cles[nbUsers - 1]);
                });
            }
        });

    },
    filSorter: function (firstUserCle, lastUserCle) {
        var userCle, newSortBy,
            urlParams = this.tabDiscoverUrlParams();
        if (urlParams.sort === 'asc') {
            userCle = lastUserCle;
            newSortBy = 'desc';
        } else {
            userCle = firstUserCle;
            newSortBy = 'asc';
        }
        var state = {
            link_id: urlParams.linkId,
            sort_by: 'created_at-' + newSortBy,
            cle: userCle
        };
        //need to show it each time order change
        // $('.btn-hide').show();
        $('.fil-sort-btn')
            .removeClass('fil-asc, fil-desc')
            .addClass('disabled fil-' + newSortBy);
        history.replaceState(state, null, 'discover.html?' + $.param(state));

        var filParams = {
            link_id: urlParams.linkId,
            user_cle: userCle
        };
        this.loadFil(filParams, function () {
            commons.search.resetSearchFilter();
        });

    },
    filActionManager: function () {
        var self = this;
        $('.sub-header .left-actions .option').click(function () {
            var $this = $(this);
            switch ($this.data('action')) {
                case 'hide':
                    self.hideFil();
                    break;
                case 'contact':
                    if (self.activeUser.has_email) {
                        bg.groupsManager.contactRequest(self.activeUser);
                    }
                    break;
            }
        });

    },
    hideFil: function () {
        var dataDiscover = this.discoverUser();
        var sort = dataDiscover.sort,
            idxUser = dataDiscover.idxUser,
            cles = this.cles;
        if (sort === 'asc') {
            if (idxUser === cles.length - 1) {
                //we reached the last
                return;
            }
            //next
            idxUser++;
        } else {
            if (idxUser === 0) {
                //we reached the first
                return;
            }
            //prev
            idxUser--;
        }

        if (!cles[idxUser]) {
            return;
        }
        // if (idxUser === 0 || (idxUser === cles.length - 1)) {
        //     $('.btn-hide').hide();
        // } else {
        //     $('.btn-hide').show();
        // }
        var userCle = cles[idxUser],
            urlParams = utils.urlParams(window.location.href),
            state = {
                link_id: urlParams.link_id,
                cle: userCle,
                sort_by: 'created_at-' + sort,
            };
        history.replaceState(state, null, 'discover.html?' + $.param(state));

        commons.search.resetSearchFilter();

        //update counter hides
        var swiper = this.swiper;
        var realIndex = swiper.realIndex;
        var idRef = $(swiper.slides[realIndex]).data('id');
        var dataCounter = {
            id_ref: idRef,
            user_cle: this.activeUser.cle, //current active
            counter: 'hides'
        };
        bg.api.counters(dataCounter);
        //load now next
        var filParams = {
            link_id: urlParams.link_id,
            user_cle: userCle
        };
        this.loadFil(filParams);
    },
    /**
     * Fetch user fil data and exec cb when success
     * @param {Object} filParams parameters to send about fil to load
     * @property {String} user_cle
     * @property {Number} linkId 
     * @property {String} filter filter name 
     * @property {String} term filter term
     */
    loadFil: function (filParams, cb) {
        var self = this;

        filParams.method = 'init';
        var $pageContent = $('.page-content');
        $('.fils-container').removeClass('ndl-active');
        $pageContent.addClass('loading');
        bg.api.getFilData(filParams, function (res) {
            utils.debug(res);
            if (res.error) {
                $pageContent.removeClass('loading');
                if (res.error === 'EMPTY_RESULT') {
                    return commons.showPageMessage(res.message, 'info');
                }
                return commons.showPageMessage(res.message || NDLConst.ERR_GLOBAL);
            }
            var user = res.user;
            self.activeUser = Object.assign({}, user);
            self.activeUser.following = res.following;
            cb && cb();
            var linkId = +filParams.link_id,
                $fil = $('.fil');
            //need to remove last fil identifier
            $fil.attr('class', function (i, c) {
                return c && c.replace(/(^|\s)fil-\S+/g, '');
            });
            //unique fil
            $fil.addClass('fil-' + user.cle);

            var initialSlide = utils.findIndexRef(res.refs, 'link_id', linkId);

            //contact button
            if (!user.has_email) {
                $('.action-left .btn-contact').hide();
                $('.left-actions .contact').removeClass('active');
            } else {
                $('.action-left .btn-contact').show();
                $('.left-actions .contact').addClass('active');
            }
            //update title
            $('.fil-header .title-text .title').text(user.title);

            $('.fils-container').addClass('ndl-active');
            var swiper = self.swiper;
            swiper.removeAllSlides();
            swiper.update();

            if (initialSlide < 0) {
                initialSlide = Math.floor(res.refs.length / 2);
            }
            var isInit = commons.initializeSwiper(res, initialSlide, swiper);
            swiper.update();
            $('.fil-sort-btn').removeClass('disabled');
            $pageContent.removeClass('loading');

        });
    },
};

$(document).ready(function () {
    discover.init();
});