var bg = chrome.extension.getBackgroundPage();
/**
 * template monfil / history / followed
 */
var template = {
    history: {
        header: function (date) {
            return $('<div class="history-header"><div class="day-date">' + date + '</div></div>');
        },
        filHeader: function (title) {
            return $(`<div class="row fil-header">
                <div class="column sixteen wide fil-title">
                    ${title} <i class="la la-external-link icon-link"></i>
                </div>
                <span class="del-history" title="Supprimer de votre historique">
                    <i class="la la-remove"></i> 
                </span>\
            </div>`);
        }
    },
    followed: {
        filHeader: function (title) {
            var filHeader = `<div class="row fil-header">
                    <div class="column sixteen wide fil-title">
                        <div class="title-text">
                            ${title} <i class="la la-external-link icon-link"></i>
                        </div>
                        <div class="ui icon input wrapper-input">
                            <input type="text" name="title" value="" autofocus maxlength="140" autocorrect="off" spellcheck="false">
                            <i class="circular checkmark link icon btn-validate-edit-title"></i>
                        </div>
                    </div>
                    <div class="fil-title-actions">
                        <span class="pen-edit-title" title="Personnaliser le titre de ce fil pour vous aider à l'identifier">
                            <i class="la la-pencil"></i>
                        </span>
                        <span class="unfollow" title="Ne plus mémoriser"><i class="la la-remove"></i></span>
                    </div>
                </div>`;
            return $(filHeader);
        }
    },
    templateRef: function (templateData) {
        var id = templateData.id,
            linkId = templateData.link_id,
            title = templateData.title,
            legende = templateData.legende,
            domain = templateData.domain,
            date = templateData.date;

        return $(`<div class="column">
            <div class="ref" data-id=${id} data-link-id=${linkId}>
                <div class="ref-icon"><i class="la"></i></div>
                <div class="ref-description">
                    <div class="link-title">${title}</div>
                    <div class="ref-infos">
                        <div class="legende">${legende}</div>
                        <div class="container-domain">
                            <span class="domain">${domain}</span>
                            <span class="date-wrapper">
                                <span class="date-msg"></span>
                                <span class="date">${date}</span>
                            </span>
                        </div>
                    </div>
                </div>
            </div>
        </div>`);
    },

};

var monFil = {
    ready: function(){
        this.init();
        this.initHistory();
        this.initFollowed();
    },
    init: function () {
        //do not init in other page (followed/history)
        if ($('body.monfil').length === 0) {
            return;
        }
        
        var userData = bg.userManager.getUser();
        var currentUser = userData.user;
        $('.pen-edit-title').click(function () {
            var $filTitle = $('.fil-title');
            monFil.filTitleUpdater($filTitle, currentUser.cle);
        });

        var urlParams = utils.urlParams(window.location.href);
        var params = {
            method: 'init',
            user_cle: currentUser.cle,

            link_id : urlParams.ref,
            filter: urlParams.filter,
            term: urlParams.term,
            selected_tags: urlParams.selected_tags
        };
        var $pageContent = $('.page-content');
        $pageContent.addClass('loading');
        var self = this;
        bg.api.myFil(params, function (res) {
            utils.debug(res);
            $pageContent.removeClass('loading');
            if (res.error === 'EMPTY_FIL') {
                return  window.location.href = '/articles.html';
            }
            if (res.error) {
                return commons.showPageMessage(res.message || NDLConst.ERR_GLOBAL);
            }
            
            var user = res.user;
            var $fil = $('.fil');
            //update fil title
            $fil.find('.fil-title .title-text').text(user.title);
            $('.fils-container').addClass('ndl-active');
            //unique fil
            $fil.addClass('fil-' + user.cle);
            
            var swiper = commons.createSwiper(user.cle);
            if (null === swiper) {
                utils.debug('ERR_INIT_SWIPER');
                return;
            }

            var linkId = urlParams.ref,
                initialSlide = -1;
            
            if(linkId){
                initialSlide = utils.findIndexRef(res.refs, 'link_id', +linkId);
            }
            if (initialSlide < 0 ) {
                initialSlide = 0;
                if(res.refs.length > 3){
                    initialSlide = res.refs.length - 2;
                }
            }
            
            linkId = res.refs[initialSlide].link_id;
            if(res.filter === 'cross'){
                $fil.data('filter', 'cross');
            }else {
                $fil.removeData('filter');
            }
            var isInit = commons.initializeSwiper(res, initialSlide, swiper);
            if(isInit){
                urlParams.cle = user.cle;
                urlParams.ref = linkId;
                //replace state here to test
                history.replaceState(urlParams, null, '?' + $.param(urlParams));
                var $searchBar = $('.searchbar');
                commons.search.init(swiper, $searchBar);
                //init cross filter
                var $checkbox = $('.checkbox-cross');
                commons.initFilterCross(swiper, $checkbox);
                console.log(res.ref_count);
                
                if(res.ref_count > 5){
                    commons.suggestUpdateFilTitle();
                }
            }
        });
    },
    /**
     * update fil title for current user fil and followed
     */
    filTitleUpdater: function ($filTitle, userCle) {
        var $actionsBtn = $filTitle.find('.fil-title-actions');
        var $wrapperInput = $filTitle.find('.wrapper-input');
        var $title = $filTitle.find('.title-text');
        var $input = $filTitle.find('input');

        var filTitle = $title.text().trim();
        filTitle = utils.removeExtraSpaces(filTitle);

        $actionsBtn.hide();
        $title.hide();
        $wrapperInput.show();
        $input.val(filTitle).focus();


        $filTitle.find('.btn-validate-edit-title').off('click').click(function () {
            var newTitle = $input.val();
            if (filTitle && newTitle && (filTitle.toLowerCase() !== newTitle.trim().toLowerCase())) {
                if (newTitle.length > 0) {
                    var data = {
                        cle: userCle,
                        title: newTitle
                    };
                    $wrapperInput.addClass('loading');
                    bg.api.updateFilTitle(data, function (res) {
                        if (!res.error) {
                            $title.text(newTitle);
                            $('.ndl-message-success').text(NDLConst.FIL_TITLE_UPDATED);
                            utils.showHideMessage('success');
                        } else {
                            $('.ndl-message-error').text(res.message || NDLConst.ERR_GLOBAL);
                            utils.showHideMessage('error');
                            $wrapperInput.removeClass('loading');
                            return;
                        }
                        $wrapperInput.removeClass('loading');
                        $wrapperInput.hide();
                        $actionsBtn.show();
                        $title.show();
                    });

                }
            } else {
                $wrapperInput.hide();
                $actionsBtn.show();
                $title.show();
            }
        });
    },
    /**
     * Group history refs by day visited
     */
    groupHistoryByDay: function (histories) {
        var groupDays = {};
        for (var i = 0, hist, daySinceEpoch; i < histories.length; i++) {
            hist = histories[i];
            daySinceEpoch = Math.floor(hist.visitedAt / (24 * 3600));
            groupDays[daySinceEpoch] = groupDays[daySinceEpoch] || [];
            groupDays[daySinceEpoch].push(hist);
        }

        return groupDays;
    },
    initHistory: function () {
        //do not init in other page (monfil/followed)
        if ($('body.history').length === 0) {
            return;
        }
        var history = bg.userManager.history.get();
        if (!history || Object.keys(history.refs).length === 0) {
            return $('body').addClass('no-history');
        }
        $('body').removeClass('no-history');
        var refs = Object.values(history.refs);
        var groupHistByDay = this.groupHistoryByDay(refs);
        var data = [];
        for (var i = 0; i < refs.length; i++) {
            data[i] = {
                user_cle: refs[i].cle,
                id_ref: refs[i].id_ref
            };
        }
        var params = {
            history: data
        };
        var self = this;
        var $pageContent = $('.page-content');
        $pageContent.addClass('loading');
        bg.api.loadHistory(params, function (res) {
            utils.debug(res);
            $pageContent.removeClass('loading');
            if (res.error) {
                if (res.error === 'ERR_NO_HISTORY') {
                    return $('body').addClass('no-history');
                }
                $('.page-global-msg')
                    .addClass('error ndl-active')
                    .find('.error-msg').html(res.message || NDLConst.ERR_GLOBAL);
                return;
            }
            self.createHistories(groupHistByDay, res.history);
        });



    },
    /**
     * Add all histories ordered by last visited
     */
    createHistories: function (groupHistByDay, filsData) {
        var sortedGroup = Object.keys(groupHistByDay);
        //sort groups in descending order to display by days desc
        sortedGroup.sort(function (d1, d2) {
            return d2 - d1;
        });
        
        for (var i = 0, day; i < sortedGroup.length; i++) {
            day = sortedGroup[i];
            var dayDate = utils.historyHeaderDate(day * 24 * 3600);
            var filsDay = groupHistByDay[day];
            //sort also fil in each group by date visted desc
            filsDay.sort(function (f1, f2) {
                return f2.visitedAt - f1.visitedAt;
            });

            var $headerHistDay = template.history.header(dayDate);
            var $filsDay = $('<div class="history-body fils"></div>');
            //create each fil in this group
            
            for (var j = 0, fil; j < filsDay.length; j++) {
                fil = filsDay[j];
                fil['refs'] = filsData[fil.cle];
                var $fil = this.createHistoryFil(fil);
                $filsDay.append($fil);
            }
            var $histDaycontainer = $('<div class="history-day"></div>');
            $histDaycontainer.append($headerHistDay)
                .append($filsDay);

            $('.history-container').append($histDaycontainer);
        }
    },
    /**
     * Create a history fil and append last visited and 
     * last added ref 
     * @param {Object} fil contains fil refs and details on last fil visite 
     */
    createHistoryFil: function (fil) {
        var refs = fil.refs,
            filTitle = refs[0].fil_title,
            userCle = refs[0].user_cle,
            // user = refs[0].user,
            $filHeader = template.history.filHeader(filTitle),
            $fil = $('<div class="ui two column grid doubling stackable fil"></div>');


        $fil.addClass('fil-' + userCle)
            .append($filHeader);
        var visitedRef = null;
        for (var i = 0, ref, $ref; i < refs.length; i++) {
            ref = refs[i];
            ref.visited_at = fil.visitedAt;
            $ref = this.createRefFil(ref);
            $fil.append($ref);
            if(!ref.is_last){
                visitedRef = ref;
            }
        }
        $filHeader.find('.del-history').click(function () {
            var $this = $(this),
                $fil = $this.closest('.fil'),
                $historyDay = $this.closest('.history-day'),
                nbFilHistoryDay = $historyDay.find('.fil').length;
            bg.userManager.history.del(userCle);
            $fil.remove();
            if (nbFilHistoryDay === 1) {
                $historyDay.remove();
            }
            if ($('.history-day').length === 0) {
                $('body').addClass('no-history');
            }
        });

        $filHeader.find('.fil-title').click(function () { 
            if(null === visitedRef){
                return;
            }            
            var ref = {
                id: visitedRef.id,
                user_cle: visitedRef.user_cle,
                link_id: visitedRef.link_id
            };
            commons.viewFil(ref);
        });

        return $fil;
    },
    /**
     * Create a fil ref for history and followed
     */
    createRefFil: function (ref) {
        var date, classRef, dateMsg;
        if (ref.is_last) {
            date = ref.created_at;
            classRef = 'last-added-ref';
            dateMsg = "Dernière ajoutée : ";
        } else {
            classRef = 'last-visited-ref';
            date = ref.visited_at;
            dateMsg = "Dernière visite : ";
        }

        var templateData = {
            id: ref.id,
            link_id: ref.link_id,
            legende: ref.legende,
            date: utils.slideShortDate(date),
            title: ref.title,
            domain: utils.hostname(ref.url),
        };
        var $ref = template.templateRef(templateData);

        if (!ref.legende || ref.legende.trim().length === 0) {
            $ref.find('.legende').remove();
        }

        $ref.find('.ref').addClass(classRef)
            .find('.date-msg').text(dateMsg);

        $ref.click(function () {
            ref.link = {
                url: ref.url
            };
            commons.browseOrDiscoverFil(ref, 'browse');
        });

        return $ref;
    },
    initFollowed: function () {
        //do not init in other page (monfil/history)
        if ($('body.followed').length === 0) {
            return;
        }
        var data = {};
        var $pageContent = $('.page-content');
        $pageContent.addClass('loading');
        var self = this;
        bg.api.loadFollowed(data, function (res) {
            utils.debug(res);
            $pageContent.removeClass('loading');
            if (res.error) {
                $('.page-global-msg')
                    .addClass('error ndl-active')
                    .find('.error-msg').html(res.message || NDLConst.ERR_GLOBAL);
                return;
            }
            var followed = res.followed;
            if (Object.keys(followed).length === 0) {
                return $('body').addClass('no-followed');
            }
            self.createFollowed(followed, res.displayOrder);
        });
    },
    /**
     * Add all followed fils ordered by last updated fil
     */
    createFollowed: function (followedFils, displayOrder) {
        $('.followed-container').addClass('ndl-active');
        var $followedFils = $('.followed-fils');
        for (var i = 0, userCle; i < displayOrder.length; i++) {
            userCle = displayOrder[i];
            const refs = followedFils[userCle];
            var $fil = this.createFollowedFil(refs);
            $followedFils.append($fil);
        }
    },
    /**
     * Create a followed fil and append last visited and 
     * last added ref 
     * @param {Array} refs array of last visited by current user
     *  and last added ref by the fil owner
     * 
     */
    createFollowedFil: function (refs) {
        var userCle = refs[0].user_cle;
        var visitedRef = refs[0].is_last ? refs[1] : refs[0];
        var $filHeader = template.followed.filHeader(visitedRef.fil_title);
        var $fil = $('<div class="ui two column grid doubling stackable fil"></div>');

        $fil.addClass('fil-' + userCle)
            .append($filHeader);
        for (var i = 0, $ref; i < refs.length; i++) {
            $ref = this.createRefFil(refs[i]);
            $fil.append($ref);
        }
        $filHeader.find('.unfollow').click(function () {
            var $this = $(this),
                $fil = $this.closest('.fil');
            var data = {
                user_cle: userCle,
                method: 'unfollow'
            };
            bg.api.userActionPost(data, function (res) {
                utils.debug(res);

                if (res.error) {
                    $('.ndl-message-error').text(res.message || NDLConst.ERR_GLOBAL);
                    utils.showHideMessage('error');
                    return;
                }
                $fil.remove();
                if (res.followed_count === 0) {
                    $('body').addClass('no-followed');
                }

            });

        });

        $filHeader.find('.pen-edit-title').click(function () {
            var $filTitle = $(this).closest('.fil-header');//.find('.fil-title');
            monFil.filTitleUpdater($filTitle, userCle);
        });
        
        $filHeader.find('.fil-title .title-text').click(function () {
            var ref = {
                id: visitedRef.id,
                user_cle: visitedRef.user_cle,
                link_id: visitedRef.link_id
            };
            commons.viewFil(ref);
        });


        return $fil;
    }
};

$(document).ready(function () {    
    commons.onPageReady(function(){
        monFil.ready();
    });    
});