"use strict";
var bg = window.chrome.extension.getBackgroundPage();

var popup = {
    showHideMessage: function (level) {
        var $popupDimmer = $('.popup-dimmer');
        $popupDimmer.addClass('active').delay(100).queue(function () {
            if (level === 'error') {
                $popupDimmer.removeClass('loading success').addClass('error');
            } else if (level === 'success') {
                $popupDimmer.removeClass('loading error').addClass('success');
            }

            setTimeout(function () {
                $popupDimmer.removeClass(level);
                //time to dissmiss message
                setTimeout(function () {
                    $popupDimmer.removeClass('active').dequeue();
                }, 500);
            }, 3000);
        });
    },
    /**
     * Update browser action when logged out
     */
    onDisconnected: function () {
        chromeAPI.getActiveTab(function (tab) {
            var iconNoFil = 'img/icons/needle-no-fil-128.png';
            chrome.browserAction.setIcon({
                path: iconNoFil,
                tabId: tab.id
            });
            chrome.browserAction.setBadgeText({
                text: "",
                tabId: tab.id
            });
        });
    },
    /**
     * Listen for incoming message 
     */
    onMessage: function () {
        chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {
            var message = request;
            switch (message.subject) {
                case 'logout':
                    $('.ndl-popup').removeClass('active');
                    $('.popup-login').addClass('active');
                    var resp = {
                        subject: message.subject,
                        data: 'logged out'
                    };
                    sendResponse(resp);
                    this.onDisconnected();
                    break;
            }
        });
    },
    init: function () {

        this.onMessage();

        if (!bg.userManager.isLoggedIn()) {
            $('.ndl-popup').removeClass('active');
            $('.popup-login').addClass('active');
            $('.btn-connexion').click(function () {
                bg.userManager.login();
                window.close();
            });
            this.onDisconnected();
            return;
        }

        this.initActionButtons();

        chromeAPI.getActiveTab(function (tab) {
            if (utils.isSupportedProtocol(tab.url)) {
                var tabData = bg.tabsManager.getTabData(tab.id, tab.url);
                var data = {
                    url: tab.url
                };
                if(tabData){
                    if(!tabData.referenced){
                        return popup.addLinkEvents(tabData, tab);
                    }
                    data.link_id = tabData.link_id;
                }
                
                var $popupDimmer = $('.popup-dimmer');
                $popupDimmer.addClass('active loading');
                
                bg.api.getLinkInfo(data, function (res) {
                    utils.debug(res);
                    if (!res.error) {
                        bg.tabsManager.setTabData(tab.id, res.data);
                        bg.tabsManager.updateBadge(tab.id);
                        popup.addLinkEvents(res.data, tab);
                        $popupDimmer.removeClass('active loading error');
                    } else {
                        $('.error').text(res.message || NDLConst.ERR_GLOBAL);
                        popup.showHideMessage('error');
                        $('.row-add-link').addClass('disabled');
                    }
                });
            } else {
                $('.row-add-link').addClass('disabled');
            }
        });
    },
    initActionButtons: function () {
        var actionUrls = {
            fil: '/monfil.html',
            group: '/groups/groups.html',
            followed: '/followed.html',
            history: '/history.html',
        };
        $('[data-action]').click(function () {
            var $this = $(this);
            var action = $this.data('action');
            var path = actionUrls[action];
            chromeAPI.isExtensionUrlOpen(path, function (tab) {
                if (tab) {
                    chrome.tabs.update(tab.id, {
                        active: true
                    });
                } else {
                    chrome.tabs.create({
                        url: chrome.runtime.getURL(path),
                        active: true
                    });
                }
                window.close();
            });

        });
    },
    /**
     * set animations and Events on buttons
     * @param {type} tabData
     * @param {type} tab
     * @returns {undefined}
     */
    addLinkEvents: function (tabData, tab) {
        var iconNoFil = 'img/popup/needle-no-fil-40.png';
        var iconWithFil = 'img/popup/needle-fil-40.png';

        var $rowAddLink = $('.row-add-link');
        var $imgNeedle = $('.img-needle img');
        var $legende = $('#legende textarea');

        var legende = tabData.legende || '';//tab.title;
        $legende.val(legende);
        if (tabData.referenced) {
            $imgNeedle.attr('src', iconWithFil);
        } else {
            $imgNeedle.attr('src', iconNoFil);
            var isNdlAnimating = false;
            $rowAddLink.mouseenter(function () {
                if (isNdlAnimating) {
                    return;
                }
                isNdlAnimating = true;
                popup.needleAnimations(function (isAnim) {
                    isNdlAnimating = isAnim;
                });
            });
        }
        var $maxCharLegendeMsg = $('.max-char-legende-msg');
        $legende.keyup(function(){
            $maxCharLegendeMsg.removeClass('active');
            $legende.removeClass('error');
            var newLegende = $legende.val();
            if(newLegende.length > 240){
                $maxCharLegendeMsg.addClass('active');
                $legende.addClass('error');
            }
            
        });

        $rowAddLink.click(function () {
            //have to get tabData each time in case ref was not added and now added
            var tabData = bg.tabsManager.getTabData(tab.id, tab.url);
            if (tabData.referenced) {
                if (tabData.local_ref > 1) {
                    popup.discover(tabData.link_id);
                }
            } else {
                var legende = $('textarea').val().trim();
                if(legende.length > 240) {
                    return;
                }
                var $popupDimmer = $('.popup-dimmer');
                $popupDimmer.addClass('active loading');
                popup.saveLink(tab, tabData, function (saved, message) {
                    var level;
                    if (saved) {
                        level = "success";
                        $('.success').text(NDLConst.REF_ADDED);
                    } else {
                        level = "error";
                        $('.error').text(message || NDLConst.ERR_GLOBAL);
                    }
                    popup.showHideMessage(level);
                });

            }
        });
        popup.updatePopupUI(tabData, tab);
    },
    /**
     * update popup UI on init and after add link
     * @param {Object} tabData
     * @param {type} tab
     * @returns {undefined}
     */
    updatePopupUI: function (tabData, tab) {
        var $message = $('.row-add-link .message');
        var $refCount = $('.row-add-link .ref-count');

        var message = "Ajouter cette page à votre fil";
        var localRef = tabData.local_ref;
        if (tabData.referenced) {
            localRef--;
            $('.row-add-link').off('mouseenter mouseleave');
            var timeouts = popup.ndlTimeouts;
            clearTimeout(timeouts.fil);
            clearTimeout(timeouts.noFil);

            var iconWithFil = 'img/popup/needle-fil-40.png';
            $('.img-needle img').attr('src', iconWithFil);

            if (localRef <= 0) {
                message = "Vous êtes le premier à avoir ajouté cette page";
            } else if (localRef === 1) {
                message = "Découvrir l'autre fil qui passe aussi sur cette page";
            } else {
                message = "Découvrir les <strong>" + localRef + "</strong>" +
                    " fils qui passent aussi sur cette page";
            }
            var $legendeAction = $('.legende-action');
            var $textarea = $('textarea');
            var $btnEdit = $('<span class="ui button btn-edit-legende">éditer</span>');
            $('.edit-legende').append($btnEdit);

            $legendeAction.addClass('active');
            $btnEdit.click(function () {
                var $this = $(this);
                var newLegende = $textarea.val().trim();
                //have to get tabData each time to check diff
                var tabData = bg.tabsManager.getTabData(tab.id, tab.url);
                var legende = tabData.legende;
                utils.debug(tabData);
                if (newLegende && newLegende.length > 0 && newLegende !== legende) {
                    newLegende = utils.removeExtraSpaces(newLegende);
                    if (newLegende === utils.removeExtraSpaces(tab.title)) {
                        return;
                    }
                    if(newLegende.length > 240) {
                        return;
                    }
                    $this.addClass('loading');
                    var data = {
                        id: tabData.id,
                        link_id: tabData.link_id,
                        legende: newLegende
                    };
                    bg.api.updateLegende(data, function (res) {
                        $this.removeClass('loading');
                        if (!res.error) {
                            if (res.updated) {
                                bg.tabsManager.updateTabData(data);
                                legende = newLegende;
                                $('.success').text(NDLConst.LEGENDE_UPDATE);
                                popup.showHideMessage('success');
                                //to update legend if monfil is open
                                var message = {
                                    subject: 'update_legende',
                                    body: data
                                };
                                bg.messaging.sendInternal(message);
                                bg.messaging.broadcast(message);
                            }
                        } else {
                            $('.error').text(res.message || NDLConst.ERR_GLOBAL);
                            popup.showHideMessage('error');
                        }
                    });
                }
            });
        } else {
            if (localRef === 1) {
                message = "<strong>" + localRef + '</strong> fil passe déjà par cette page,' +
                    " pour le voir ajoutez-la à votre fil";
            } else if (localRef > 1) {
                message = "<strong>" + localRef + '</strong> fils passent déjà par cette page,' +
                    " pour les voir ajoutez-la à votre fil";
            }
            //change font size to contain text in 2 lines
            $message.closest('.text').addClass('long');
        }
        $message.html(message);
        $refCount.html((localRef > 0 ? localRef : ''));
    },
    //make global to stop animation timer when saving a link
    ndlTimeouts: {
        fil: 0,
        noFil: 0
    },
    needleAnimations: function (cb) {
        var $imgNeedle = $('.img-needle img');
        var timeouts = this.ndlTimeouts;
        timeouts.fil = setTimeout(function () {
            $imgNeedle.attr('src', "img/popup/needle-anime.gif");
            //remove gif after animation
            timeouts.noFil = setTimeout(function () {
                $imgNeedle.attr('src', "img/popup/needle-no-fil-40.png");
                clearTimeout(timeouts.fil);
                clearTimeout(timeouts.noFil);
                cb(false);
            }, 1200);
        }, 500);
    },
    /**
     * Save a link to remote
     * @param {type} tab
     * @param {Object} tabData loaded data containing localRef link before
     * request save link
     * @param {type} cb
     * @returns {undefined}
     */
    saveLink: function (tab, tabData, cb) {
        var legende = $('textarea').val().trim();
        legende = (tab.title === legende) ? null : legende;
        var linkData = {
            title: tab.title,
            url: tab.url,
            legende: legende,
            local_ref: tabData.local_ref,
            link_id: tabData.link_id
        };
        bg.api.saveLink(linkData, function (res) {
            utils.debug('save link ', res);
            if (!res.error) {
                bg.tabsManager.setTabData(tab.id, res.data);
                bg.tabsManager.updateBadge(tab.id);
                popup.updatePopupUI(res.data, tab);

                //other users had added this link open discover page
                if (res.data.local_ref > 1) {
                    setTimeout(function () {
                        popup.discover(res.data.link_id);
                    }, 1000);
                }
                //notify new ref is added to rebuild slider dates
                var data = {
                    user_cle: res.user_cle,
                    dates: res.dates
                };
                var message = {
                    subject: 'update_slider_dates',
                    body: data
                };
                bg.messaging.sendInternal(message);
                bg.messaging.broadcast(message);
            }
            cb(!res.error, res.message);
        });
    },
    discover: function (linkId) {
        var path = '/discover.html';
        var urlParams = {
            link_id: linkId,
            sort_by: 'created_at-asc'
        };
        var urlDiscoverPage = chrome.runtime.getURL(
            '/discover.html?' + $.param(urlParams));
        chromeAPI.isExtensionUrlOpen(path, function (tabDiscover) {
            if (tabDiscover) {
                chrome.tabs.update(tabDiscover.id, {
                    url: urlDiscoverPage,
                    active: true
                });
            } else {
                chrome.tabs.create({
                    url: urlDiscoverPage,
                    active: true
                });
            }
            window.close();
        });
    }
};

$(document).ready(function () {
    popup.init();

});