/**
 * Enable/ disable debugging
 */

// extend Storage API to set and get JS objects
Storage.prototype.setObject = function (key, value) {
    this.setItem(key, JSON.stringify(value));
};
Storage.prototype.getObject = function (key) {
    var value = this.getItem(key);
    return value && JSON.parse(value);
};
const DEBUG = false,
    HTML_ENTITIES = {
        '&': '&amp;',
        '>': '&gt;',
        '<': '&lt;',
        '"': '&quot;',
        "'": '&#39;'
    },
    COLORS_PRRIORITIES = {
        me: 0,
        memorized: 1,
        filet: 2,
        history: 3,
        currentServer: 4,
        hasProfil: 5,
        anonymous: 6
    },
    COLORS = [
        {
            priority: 0,
            color: 'teal',
            hex: '#00B2B2',
            desc: 'me'
        },
        {
            priority: 1,
            color: 'purple',
            hex: '#096CA8',
            desc: 'memorized_or_followed'
        },
        {
            priority: 2,
            color: 'green',
            hex: '#9BC115',
            desc: 'filet_or_group'
        },
        {
            priority: 3,
            color: 'blue',
            hex: '#2217A8',
            desc: 'history'
        },
        {
            priority: 4,
            color: 'yellow',
            hex: '#E29225',
            desc: 'current_server'
        },
        {
            priority: 5,
            color: 'orange',
            hex: '#DD582B',
            desc: 'has_profil'
        },
        {
            priority: 6,
            color: 'gray',
            hex: '#353526',
            desc: 'anonymous'
        },
    ];

/**
 * utility for extension and content script
 */
var utils = {
    /**
     * Function for enabling/disabling console log
     */
    debug: (function () {
        if (DEBUG) {
            return console.log.bind(window.console);
        }
        return function () {};
    })(),
    frWeekDays: ["Dimanche", "Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi"],
    frWeedkDaysShort: ["Dim.", "Lun.", "Mar.", "Mer.", "Jeu.", "Ven.", "Sam."],
    frMonthShort: ["Janv.", "Févr.", "Mars", "Avr.", "Mai", "Juin", "Juil.", "Août", "Sept.", "Oct.", "Nov.", "Déc."],
    frMonth: ["Janvier", "Février", "Mars", "Avril", "Mai", "Juin", "Juillet", "Août", "Septembre", "Octobre", "Novembre", "Décembre"],
    filColorsClasses: 'teal purple green blue yellow orange gray',
    /**
     * Time since a giving date
     * @param {String | Number} timestampOrStrDate timestamp in seconds or date string
     */
    _timeSince: function (timestampOrStrDate) {
        var timestamp;
        if (typeof timestampOrStrDate !== "number") {
            timestamp = Math.floor((new Date(timestampOrStrDate)).getTime() / 1e3);
        } else {
            timestamp = timestampOrStrDate;
        }

        var tsNow = Math.floor((new Date).getTime() / 1e3);
        var seconds = tsNow - timestamp;
        if (seconds < 1) { //less than a second
            return "A l'instant";
        }
        if (seconds < 60) { //les than a minute
            return 'il y a ' + seconds + ' sec' + (seconds > 1 ? 's' : '');
        }

        var interval = Math.floor(seconds / 60);
        if (interval < 60) { //less than hour
            return 'il y a ' + interval + ' min' + (interval > 1 ? 's' : '');
        }
        interval = Math.floor(seconds / 3600);
        if (interval < 24) { // less than a day
            return 'il y a ' + interval + ' heure' + (interval > 1 ? 's' : '');
        }
        interval = Math.floor(seconds / 86400);
        if (interval === 1) {
            return 'hier';
        }
        if (interval <= 6) {
            return 'il y a ' + interval + ' jour' + (interval > 1 ? 's' : '');
        }
        return timestamp;
        // var date = new Date(1e3 * timestamp);
        // var strDate = this._leadingZero(date.getDate()) + " " +
        //     this.frMonth[date.getMonth()] + " " +
        //     date.getFullYear().toString().substr(-2);

        // return strDate; //return date format dd mm yy (ex: 14 Décembre 17)
    },
    _leadingZero: function (val) {
        return val < 10 ? "0" + val : val;
    },
    /**
     * Return Date object
     * @param {Number | String} timestampOrStrDate timestamp in second or 
     * date string
     */
    _getDate(timestampOrStrDate) {
        if (typeof timestampOrStrDate !== "number") {
            return new Date(timestampOrStrDate);
        }
        return new Date(1e3 * timestampOrStrDate);
    },
    /**
     * Convert date to ISO string
     */
    toISOString: function (timestampOrStrDate) {
        if (!timestampOrStrDate) {
            return;
        }
        var date = this._getDate(timestampOrStrDate);
        return date.toISOString();
    },
    /**
     * Return the date part as string or number
     * @param {Date} date 
     * @param {Boolean} toNumber return date part as string or number ?
     */
    datePart: function (date, toNumber) {
        var day = date.getDate();
        var month = date.getMonth();
        var year = date.getFullYear();
        if (toNumber) {
            return [year, month, day];
        }
        return [year, this._leadingZero(month + 1), this._leadingZero(day)];
    },
    /**
     * Format date according the lang
     * YYYY-mm-dd or dd/mm/YYYY
     */
    formatLang: function (timestampOrStrDate, lang) {
        var s = this.datePart(this._getDate(timestampOrStrDate));
        if (lang === 'en') {
            return s[0] + "-" + s[1] + "-" + s[2];
        }
        return s[2] + "/" + s[1] + "/" + s[0];
    },
    slideShortDate: function (timestampOrStrDate) {
        var time = this._timeSince(timestampOrStrDate);
        if (typeof time === 'number') {
            var date = new Date(1e3 * time);
            var strDate = this._leadingZero(date.getDate()) + " " +
                this.frMonth[date.getMonth()] + " " +
                date.getFullYear().toString().substr(-2);
            return strDate;
        }
        return this._timeSince(timestampOrStrDate);
    },
    historyHeaderDate: function (timestampOrStrDate) {
        var date = this._getDate(timestampOrStrDate),
            s = this.datePart(date, true),
            day = s[2],
            month = s[1],
            year = s[0];
        var weekDay = date.getDay();
        return this.frWeekDays[weekDay] + ", " + day + ' ' + this.frMonth[month] + " " + year;
    },
    sliderDate: function (timestampOrStrDate) {
        var time = this._timeSince(timestampOrStrDate);
        if (typeof time === 'number') {
            var date = new Date(1e3 * time);
            var strDate = this._leadingZero(date.getDate()) + " " +
                this.frMonthShort[date.getMonth()] + " " +
                date.getFullYear().toString().substr(-2);
            return strDate;
        }
        return this._timeSince(timestampOrStrDate);
    },
    articleDates: function (timestampOrStrDate) {
        var time = this._timeSince(timestampOrStrDate);
        if (typeof time === 'number') {
            var date = new Date(1e3 * time);
            var strDate = this._leadingZero(date.getDate()) + " " +
                this.frMonth[date.getMonth()] + " " +
                date.getFullYear();
            return strDate;
        }
        return this._timeSince(timestampOrStrDate);
    },
    /**
     * Giving an attribute and val to search , reutrn its index in the array
     * (Allow determining initialSlides) 
     * @param {Array} refs array of object
     * @param {String} attr attr to compare to val
     * @param {any} val the value
     */
    findIndexRef: function (arr, attr, val) {
        return arr.findIndex(function (el) {
            return el[attr] === val;
        });
    },
    isNumeric: function (n) {
        return !isNaN(parseFloat(n)) && isFinite(n);
    },
    /**
     * How many percent is this value of this range?
     * @param {array} sliderRange [min-max] of the slider
     * @param {number} value the value to get position in percentage
     * @returns value pourcentage 
     */
    fromIndexToPercentage: function (sliderRange, value) {
        return (value * 100) / (sliderRange[1] - sliderRange[0]);
    },
    /**
     * Test if url start with http|https| ftp | ftps
     * @param {string} url
     * @returns {boolean}
     */
    isSupportedProtocol: function (url) {
        return /^(f|ht)tps?/i.test(url);
    },
    /**
     * Retrieve hostname from an url
     * @param {type} url
     * @returns {.utils@call;parseUri.host}
     */
    hostname: function (url) {
        var parsedUri = utils.parseUri(url);
        return parsedUri.host;
    },
    showHideMessage: function (level) {
        if (level === 'error') {
            $('.ndl-message.ndl-message-error').addClass('show');
        } else if (level === 'success') {
            $('.ndl-message.ndl-message-success').addClass('show');
        }
        setTimeout(function () {
            $('.ndl-message').removeClass('show');
        }, 5000);
    },
    /**
     * Normalize url by removing hash and trailing slash from it,
     * replace https to http to allow comparaison
     * @param {String} url
     */
    normalizeUrl: function (url) {
        if (!url) {
            return null;
        }
        url = url.split('#')[0]; //remove the hash
        while (url[url.length - 1] === '/') {
            url = url.slice(0, -1);
        }
        //replace at the begining ignore case
        return url.replace(/^(https?)/i, 'http').replace(/^(ftps?)/i, 'ftp');
    },
    /**
     * Get url query strings as object
     * @param {string} url
     * @returns Object
     */
    urlParams: function (url) {
        var params = url.split('?')[1];
        return params ? deparam(params) : {};
        // var parsedUri = utils.parseUri(url);
        // return parsedUri.queryKey;
    },
    /**
     * Remove attr with no value in params
     * @param {Object} params
     * @returns Object 
     */
    cleanUrlParams: function (params) {
        var keys = Object.keys(params);
        for (var i = 0, key, val; i < keys.length; i++) {
            key = keys[i];
            val = params[key];
            if (!val || val.length === 0) {
                delete params[key];
            }
        }
        return params;
    },
    /**
     * Remove extras space in a string, (fil title for exple)
     * @param {String} str the string
     */
    removeExtraSpaces: function (str) {
        return str.replace(/\s\s+/g, ' ');
    },
    htmlEscape: function (text) {
        return text && text.replace(/[&"'><]/g, function (character) {
            return HTML_ENTITIES[character];
        });
    },
    /**
     * make hashtags in text as link
     */
    autoLinkHashtags: function (text) {
        if (!text || text.trim().length === 0) {
            return text;
        }
        var twttrTxt = twttr.txt;
        var hashtagsWithIndices = twttrTxt.extractHashtagsWithIndices(text);
        var result = '';
        if (hashtagsWithIndices.length > 0) {
            var extracted, hashtag, indices, startIndex = 0;
            for (var i = 0; i < hashtagsWithIndices.length; i++) {
                extracted = hashtagsWithIndices[i];

                hashtag = extracted.hashtag;
                indices = extracted.indices;

                result += text.substring(startIndex, indices[0]);
                result += '<a class="ndl-hashtag">#' + hashtag + '</a>';
                startIndex = indices[1];
            }
            result += text.substring(startIndex, text.length);
            return result;
        }
        return text;
    },
    /**
     * Get user initials name (2 letters) for fil title
     */
    getInitials: function(name){
        name = name || '';
        name = name.trim().length > 0 ? name : "Fil sans titre";
        var parts = name.match(/\b\w/g) || [];
        var initials = "";
        for (var i = 0; i < 2; i++) {
            initials += parts[i] || '';
        }
        return initials.toUpperCase();
    },
    /**
     * Return a user fil color
     * @param {array} colorsPriority array integer without history priority
     * @param {string} userCle user cle to return fil color
     * @param {object} historyFromCs if fil color is requested by content script,
     * it must send history object
     */
    getFilColor: function(colorsPriorities, userCle, historyFromCs){
        if(colorsPriorities.length === 0){
            return {};
        }
        //add history priority first in colorsPriorities
        var history;
        if(historyFromCs) {
            history = historyFromCs;
        }else {
            history = localStorage.getObject('userHistory') || {};
        }

        var refsHistory = history.refs || {},
            userClesInHistory = Object.keys(refsHistory);
        
        if(userClesInHistory.length > 0){
            //remove histoy priority if present to avoid dupicate
            var posHistoryPriority = colorsPriorities.indexOf(COLORS_PRRIORITIES.history);
            if(posHistoryPriority > -1){
                colorsPriorities.splice(posHistoryPriority, 1);
            }
            if(userClesInHistory.indexOf(userCle) > -1) {
                colorsPriorities.push(COLORS_PRRIORITIES.history);
            }
        }
        colorsPriorities = colorsPriorities.sort(function(a, b){
            return a - b;
        });
        var idxColor = colorsPriorities[0];
        return COLORS[idxColor];

    },
    parseUri: parseUri
};

//parseUri from : https://gist.github.com/szepeviktor/8283110
function parseUri(url) {
    var o = parseUri.options,
        m = o.parser[o.strictMode ? "strict" : "loose"].exec(url),
        uri = {},
        i = 14;

    while (i--)
        uri[o.key[i]] = m[i] || "";

    uri[o.q.name] = {};
    uri[o.key[12]].replace(o.q.parser, function ($0, $1, $2) {
        if ($1)
            uri[o.q.name][$1] = $2;
    });

    return uri;
}

parseUri.options = {
    strictMode: false,
    key: ["source", "protocol", "authority", "userInfo", "user", "password", "host", "port", "relative", "path", "directory", "file", "query", "anchor"],
    q: {
        name: "queryKey",
        parser: /(?:^|&)([^&=]*)=?([^&]*)/g
    },
    parser: {
        strict: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,
        loose: /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/
    }
};


/**
 * Modified: Removed jquery in the method
 * jQuery deparam is an extraction of the deparam method from Ben Alman's jQuery BBQ
  http://benalman.com/projects/jquery-bbq-plugin/
  https://github.com/chrissrogers/jquery-deparam/blob/master/jquery-deparam.js
 */
/*
  jQuery deparam is an extraction of the deparam method from Ben Alman's jQuery BBQ
  http://benalman.com/projects/jquery-bbq-plugin/
*/
function deparam(params, coerce) {
    var obj = {},
        coerce_types = {
            'true': !0,
            'false': !1,
            'null': null
        };
    var NameValues = params.replace(/\+/g, ' ').split('&');

    // Iterate over all name=value pairs.
    NameValues.map(function (v, j) {
        // $.each(params.replace(/\+/g, ' ').split('&'), function (j, v) {
        var param = v.split('='),
            key = decodeURIComponent(param[0]),
            val,
            cur = obj,
            i = 0,

            // If key is more complex than 'foo', like 'a[]' or 'a[b][c]', split it
            // into its component parts.
            keys = key.split(']['),
            keys_last = keys.length - 1;

        // If the first keys part contains [ and the last ends with ], then []
        // are correctly balanced.
        if (/\[/.test(keys[0]) && /\]$/.test(keys[keys_last])) {
            // Remove the trailing ] from the last keys part.
            keys[keys_last] = keys[keys_last].replace(/\]$/, '');

            // Split first keys part into two parts on the [ and add them back onto
            // the beginning of the keys array.
            keys = keys.shift().split('[').concat(keys);

            keys_last = keys.length - 1;
        } else {
            // Basic 'foo' style key.
            keys_last = 0;
        }

        // Are we dealing with a name=value pair, or just a name?
        if (param.length === 2) {
            val = decodeURIComponent(param[1]);

            // Coerce values.
            if (coerce) {
                val = val && !isNaN(val) ? +val // number
                    :
                    val === 'undefined' ? undefined // undefined
                    :
                    coerce_types[val] !== undefined ? coerce_types[val] // true, false, null
                    :
                    val; // string
            }

            if (keys_last) {
                // Complex key, build deep object structure based on a few rules:
                // * The 'cur' pointer starts at the object top-level.
                // * [] = array push (n is set to array length), [n] = array if n is 
                //   numeric, otherwise object.
                // * If at the last keys part, set the value.
                // * For each keys part, if the current level is undefined create an
                //   object or array based on the type of the next keys part.
                // * Move the 'cur' pointer to the next level.
                // * Rinse & repeat.
                for (; i <= keys_last; i++) {
                    key = keys[i] === '' ? cur.length : keys[i];
                    cur = cur[key] = i < keys_last ?
                        cur[key] || (keys[i + 1] && isNaN(keys[i + 1]) ? {} : []) :
                        val;
                }

            } else {
                // Simple key, even simpler rules, since only scalars and shallow
                // arrays are allowed.

                if ($.isArray(obj[key])) {
                    // val is already an array, so push on the next value.
                    obj[key].push(val);

                } else if (obj[key] !== undefined) {
                    // val isn't an array, but since a second value has been specified,
                    // convert val into an array.
                    obj[key] = [obj[key], val];

                } else {
                    // val is a scalar.
                    obj[key] = val;
                }
            }

        } else if (key) {
            // No value was defined, so set something meaningful.
            obj[key] = coerce ?
                undefined :
                '';
        }
    });

    return obj;
}